# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
"""Routes configuration

The more specific and detailed routes should be defined first so they
may take precedent over the more generic routes. For more information
refer to the routes manual at http://routes.groovie.org/docs/
"""
from pylons import config
from routes import Mapper

def make_map():
    """Create, configure and return the routes Mapper"""
    map = Mapper(directory=config['pylons.paths']['controllers'],
                 always_scan=config['debug'])

    # The ErrorController route (handles 404/500 error pages); it should
    # likely stay at the top, ensuring it can always be resolved
    map.connect('error/:action/:id', controller='error')

    # CUSTOM ROUTES HERE
    # TAKE CARE OF ORDER HERE!!! FIRST RULE THAT MATCH WILL BE USED!
    map.connect('', controller='waska', action='index')
    map.connect('document/caseDelete/:id/:case/:confirmed', action="caseDelete", controller="document") 
    map.connect('rg_aid/delete/:rg_id/:ds_type/:confirmed', action="delete", controller="rg_aid") 
    map.connect('case/markForDelete/:id/:confirmed', controller="case", action="markForDelete")
    map.connect('case/deleteAppointment/:id/:confirmed', controller="case", action="deleteAppointment")
    map.connect('case/deleteLogbookEntry/:id/:confirmed', controller="case", action="deleteLogbookEntry")
    map.connect('case/delete/:id/:confirmed', controller="case", action="delete")
    map.connect('case/restore/:id/:confirmed', controller="case", action="restore")
    map.connect('case/open/:id/:confirmed', controller="case", action="open")
    map.connect('case/close/:id/:confirmed', controller="case", action="close")
    map.connect('case/select/:id/:confirmed', controller="case", action="select")
    map.connect('case/keepActive/:id/:confirmed', controller="case", action="keepActive")
    map.connect('case/phase/:id/:confirmed', controller="case", action="phase")
    map.connect('case/printAll/:id/:empty', controller="case", action="printAll", empty=0)
    map.connect('case/showAppointment/:id/:confirmed', controller="case", action="showAppointment", confirmed=1)
    map.connect('case/anonymize/:id/:confirmed', controller="case", action="anonymize")
    map.connect('case/markForAnonymize/:id/:confirmed', controller="case", action="markForAnonymize")
    map.connect('case/markForAnonymizeFromOverview/:id/:confirmed', controller="case", action="markForAnonymizeFromOverview")
    map.connect('case/logbook/:id/:sort_field/:sort_order', controller="case", action="logbook") #sorting of logbookentries
    map.connect('case/:action/:ds_id/:page_id/:form', controller="case") 
    map.connect('case/:action/:ds_id/:page_id', controller="case")
    map.connect('navigation/:action/:ds_id/:page_id', controller="navigation")
    map.connect('rg_aid/:action/:rg_id/:page_id/:form', controller="rg_aid") 
    map.connect('rg_aid/:action/:rg_id/:page_id', controller="rg_aid")
    map.connect('formhelp', 'case/showhelp/:help_id', controller="case", action="showHelp")
    map.connect('required', 'case/required/:help_id', controller="case", action="showRequired")
    map.connect('showevalhelp', 'case/showHelpEvaluation/:help_id', controller="case", action="showHelpEvaluation")
    map.connect('digest', 'case/digest/:id', controller="case", action="digest")
    map.connect('case/logbook/:id/:sort_field/:sort_order', controller="case", action="logbook", sort_field='datum', sort_order='desc') #sorting of logbookentries
    map.connect('case_overview/overview/:reset/:sort_field/:sort_order', controller="case_overview", action="overview", reset=0, sort_field="name_1", sort_order="asc") #for bundles actions in case_overview
    map.connect('case_overview/:action/:confirmed', controller="case_overview") #for bundles actions in case_overview
    map.connect('agency_overview/overview/:reset/:sort_field/:sort_order', controller="agency_overview", action="overview", reset=0, sort_field="fkz", sort_order="asc") #for bundles actions in agency_overview
    map.connect('agency_overview/overview/:reset', controller="agency_overview", action="overview") #for bundles actions in agency_overview
    map.connect('evaluate/evaluate/:id/:reset', controller="evaluate", action="evaluate") #for evaluation over all cases.
    map.connect(':controller/:action/:id/:confirmed')
    map.connect(':controller/:action/:id')
    map.connect(':controller/:action/*(file).csv')
    map.connect('*url', controller='template', action='view')

    return map
