# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
# Sascha L. Teichmann <teichmann@intevation.de>
#

import logging
import formencode

from formencode                    import htmlfill
from waskaweb.lib.base             import *
from waskaweb.lib.security         import getDbName 
from waskaweb.lib.validators       import ValidatorStateObject, NewUserForm, EditUserForm, NewPasswordForm, EditSettingsForm, DeleteUserHelperForm, NewUserGroupForm, EditUserGroupForm
from waskaweb.model.user           import UserObject, UserListObject, UserGroup, UserGroupList, UserGroupFactory
from waskaweb.model.case           import CaseOverview, CaseBundle 
from waskaweb.model.agencysettings import Agency
from waskaweb.lib.security         import checkRole

log = logging.getLogger(__name__)

RESET_PASSWORD_CONFIRM                  = _('adm_reset_user_password')
RESET_PASSWORD_CONFIRM_QUESTION         = _('adm_reset_user_password_question') 

DELETE_USER_NOTIFICATION_SUCCESS        = _('adm_delete_user_success_header')
DELETE_USER_NOTIFICATION_TEXT_SUCCESS   = _('adm_delete_user_success_text')

DELETE_SELFUSER_FAILURE                 = _('adm_delete_selfuser_notification_header')
DELETE_SELFUSER_FAILURE_TEXT            = _('adm_delete_selfuser_notification_text')
DELETE_USER_NOTIFICATION                = _('adm_delete_user_notification_header')
DELETE_USER_NOTIFICATION_TEXT           = _('adm_delete_user_notification_text')
NEW_USER_FAILURE_HEADER                 = _('adm_delete_user_failure_header')
NEW_USER_FAILURE_TEXT                   = _('adm_delete_user_failure_text')
DELETE_USER_SUCCESS_HEADER              = _('adm_delete_user_success_header')
DELETE_USER_SUCCESS_TEXT                = _('adm_delete_user_success_text')

NEW_USERGROUP_SUCCESS_HEADER            = _('adm_new_usergroup_success_header')
NEW_USERGROUP_SUCCESS_TEXT              = _('adm_new_usergroup_success_text')
NEW_USERGROUP_FAILURE_HEADER            = _('adm_new_usergroup_failure_header')
NEW_USERGROUP_FAILURE_TEXT              = _('adm_new_usergroup_failure_text')
EDIT_USERGROUP_SUCCESS_HEADER           = _('adm_edit_usergroup_header')
EDIT_USERGROUP_SUCCESS_TEXT             = _('adm_edit_usergroup_text')
DELETE_USERGROUP_CONFIRM                = _('adm_delete_usergroup_confirm_header')
DELETE_USERGROUP_CONFIRM_QUESTION       = _('adm_delete_usergroup_confirm_text')
DELETE_USERGROUP_SUCCESS_HEADER         = _('adm_delete_usergroup_success_header')
DELETE_USERGROUP_SUCCESS_TEXT           = _('adm_delete_usergroup_success_text')
DELETE_USERGROUP_FAILED_HEADER          = _('adm_delete_usergroup_failure_header')
DELETE_USERGROUP_FAILED_TEXT            = _('adm_delete_usergroup_failure_text')

DELETE_USER_CONFIRM                     = _('adm_delete_user_confirm_header')
DELETE_USER_CONFIRM_QUESTION            = _('adm_delete_user_confirm_text')

DIALOG_CONFIRM_SETEDITOR                = _('adm_set_editor_header')
SETEDITOR_NOTIFICATION_SUCCESS          = _('adm_set_editor_success_text')
SETEDITOR_NOTIFICATION_SUCCESS_REST     = _('adm_set_editor_success_text_rest')
SETEDITOR_NOTIFICATION_SUCCESS_HEAD     = _('adm_set_editor_success_text_head')
SETEDITOR_NOTIFICATION_SUCCESS_REST_HEAD= _('adm_set_editor_success_text_rest_head')

EDIT_USER_SUCCESS_HEADER                = _('adm_edit_user_header')
EDIT_USER_SUCCESS_TEXT                  = _('adm_edit_user_text')

EDIT_AGENCY_SUCCESS_HEADER              = _('adm_edit_agency_header')
EDIT_AGENCY_SUCCESS_TEXT                = _('adm_edit_agency_text')

class AdministrationController(BaseController):

    def index(self):
        redirect_to(action='overviewUser')

    @checkRole(('admin_ka', 'cm_ka', 'pt_dlr'))
    def accountDeactivated(self):
        try:
            del session['USER_AUTHORIZED']
            session.save()
        except KeyError:
            pass
        return render('/administration/account_deactivated.mako')

    @checkRole('admin_ka')
    def overviewUser(self):
        list = UserListObject()
        c.user_overview = list.getUserList()
        return render('/administration/overview_user.mako')

    @checkRole('admin_ka')
    def overviewUserGroups(self):
        # Get gid of usergroups so we can filter them out.
        user             = UserListObject()
        filter           = [u.gid for u in user.getUserList()]

        list             = UserGroupList()
        c.group_overview = list.getGroupList(filter)
        return render('/administration/overview_usergroups.mako')

    @checkRole('admin_ka')
    def showUser(self, id=None):
        id = self._checkInt(id)
        c.uo = UserObject(id)
        c.vuo = UserGroup(c.uo.standin)
        return render('/administration/show_user.mako')

    @checkRole('admin_ka')
    def showUserGroup(self, id=None):
        factory = UserGroupFactory()
        id = self._checkInt(id)
        group = factory.load(id)
        groupmember = group.getUser()
        c.group = group
        c.groupmember = groupmember
        return render('/administration/show_usergroup.mako')

    @checkRole('admin_ka')
    def editUserGroup(self, id=None):
        id            = self._checkInt(id)
        factory       = UserGroupFactory()
        group         = factory.load(id)
        memberlist    = group.getUser().getUserList()
        c.form_errors = {}
        c.form_result = {'name': group.name,
                         'id': group.id,
                         'standin': [user.id for user in memberlist]}
        c.group       = group
        userlist      = UserListObject()
        admins        = userlist.getAdminList()
        c.userlist    = userlist.getUserList(filter = [u.id for u in admins])
        form = render('/administration/edit_usergroup.mako')
        return htmlfill.render(unicode(form, 'utf-8'),
                               c.form_result,
                               c.form_errors,
                               auto_insert_errors=False)

    @checkRole('admin_ka')
    def editUserGroupAction(self):
        validator   = EditUserGroupForm()
        factory     = UserGroupFactory()
        userlist    = UserListObject()
        admins      = userlist.getAdminList()
        c.userlist  = userlist.getUserList(filter = [u.id for u in admins])
        try:
            try:
                state = ValidatorStateObject
                data = formencode.variabledecode.variable_decode(request.params)
                state.groupid = data.get('id')
                form_result = validator.to_python(data, state)
                group = factory.load(form_result.get('id'))
                group.setData(form_result)
                group.setUser(form_result.get('standin'))
                group.store()
                c.group         = group
                c.success_for   = EDIT_USERGROUP_SUCCESS_HEADER
                c.success_text  = EDIT_USERGROUP_SUCCESS_TEXT
                c.url_ok        = "/administration/overviewUserGroups"
                return render('/administration/dialogs/success_edit_usergroup.mako')
            except formencode.Invalid, error:
                c.form_result = error.value
                c.group = factory.load(c.form_result.get('id'))
                c.form_errors = error.error_dict or {}
                form = render('/administration/edit_usergroup.mako')
                return htmlfill.render(unicode(form, 'utf-8'),
                                       c.form_result,
                                       c.form_errors,
                                       auto_insert_errors=False)
        except:
            return render('/tests/trace.mako')

    @checkRole('admin_ka')
    def editUser(self, id=None):
        id = self._checkInt(id)
        c.form_errors = {}
        c.form_result = {}
        c.uo = UserObject(id)
        return render('/administration/edit_user.mako') 

    @checkRole('admin_ka')
    def resetUserPassword(self, id=None, confirmed='0'):
        id = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        c.uo = UserObject(id)
        if confirmed:
            try:
                c.new_password = c.uo.resetPassword()
                return render('/administration/reset_password.mako')
            except:
                return render('/tests/trace.mako')
        else:
            c.context = "../main.mako" 
            c.confirm_for = RESET_PASSWORD_CONFIRM 
            c.question = "%s %s, %s" % (RESET_PASSWORD_CONFIRM_QUESTION, c.uo.last_name, c.uo.first_name) 
 
            c.url_yes = "/administration/resetUserPassword/%s/1" % id
            c.url_no  = "/administration/showUser/%s" % id
            return render('/administration/dialogs/confirm_resetpassword.mako')

    @checkRole('admin_ka')
    def newUser(self):
        c.form_errors = {}
        c.form_result = {}
        return render('/administration/new_user.mako')

    @checkRole('admin_ka')
    def newUserGroup(self):
        c.form_errors = {}
        c.form_result = {}
        return render('/administration/new_usergroup.mako')

    @checkRole('admin_ka')
    def editUserAction(self):
        validator = EditUserForm() 
        try:
            try:
                uid = request.params['uid']
                c.uo = UserObject(uid)
                form_result = validator.to_python(request.params)
                c.uo.setData(form_result)
                c.uo.store()
                c.success_for   = EDIT_USER_SUCCESS_HEADER
                c.success_text  = EDIT_USER_SUCCESS_TEXT
                c.url_ok        = "/administration/overviewUser"
                return render('/administration/dialogs/success_edit_user.mako')
                #return self.overviewUser() 
            except formencode.Invalid, error:
                c.form_result = error.value
                c.form_errors = error.error_dict or {}
                return render('/administration/edit_user.mako')
        except:
            return render('/tests/trace.mako')
            #return "Es ist ein Fehler beim Speichern des Benutzers aufgetreten."

    @checkRole('admin_ka')
    def createUserGroupAction(self):
        factory   = UserGroupFactory()
        validator = NewUserGroupForm() 
        try:
            try:
                c.form_result  = validator.to_python(request.params)
                c.ugo          = factory.create(c.form_result.get('name'))
                c.success_for  = NEW_USERGROUP_SUCCESS_HEADER
                c.success_text = NEW_USERGROUP_SUCCESS_TEXT
                c.url_ok       = "/administration/overviewUserGroups"
                return render('/administration/dialogs/success_new_usergroup.mako')
            except formencode.Invalid, error:
                c.form_result = error.value
                c.form_errors = error.error_dict or {}
                form = render('/administration/new_usergroup.mako')
                return htmlfill.render(unicode(form, 'utf-8'), 
                                       c.form_result,
                                       c.form_errors,
                                       auto_insert_errors=False)
            except:
                c.failed_for   = NEW_USERGROUP_FAILURE_HEADER
                c.failed_text  = NEW_USERGOUP_FAILURE_TEXT
                c.url_ok       = "/administration/overviewUserGroups"
                return render('/administration/dialogs/failed_new_usergroup.mako')
        except:
            return render('/tests/trace.mako')

    @checkRole('admin_ka')
    def createUserAction(self):
        validator = NewUserForm() 
        c.user_password = None
        try:
            try:
                agency = getDbName()
                state = ValidatorStateObject
                state.agency = agency
                c.form_result = validator.to_python(request.params, state)
                c.uo = UserObject()
                # Set initial data which is needed to create a new user
                login = c.form_result.get('login', '')
                role = c.form_result.get('role', '')
                c.uo.setData(c.form_result)
                # create user
                c.user_password = c.uo.createLogin(login, role, agency)
                return render('/administration/new_user_result.mako') 
            except formencode.Invalid, error:
                c.form_result = error.value
                c.form_errors = error.error_dict or {}
                form = render('/administration/new_user.mako')
                return htmlfill.render(unicode(form, 'utf-8'), c.form_result, c.form_errors, auto_insert_errors=False)
            except:
                c.failed_for = NEW_USER_FAILURE_HEADER 
                c.failed_text  = NEW_USER_FAILURE_TEXT 
                return render('/tests/trace.mako')
                c.url_ok     = "/administration/overviewUser"
                return render('/administration/dialogs/failed_new_user.mako')
                
        except:
            return render('/tests/trace.mako')
            #return "Es ist ein Fehler beim Speichern des Benutzers aufgetreten."

    @checkRole('admin_ka')
    def deleteUserGroup(self, id, confirmed='0'):
        id = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        factory = UserGroupFactory()
        try:
            ugo = factory.load(id)
            if confirmed:
                ugo.delete()
                c.success_for  = DELETE_USERGROUP_SUCCESS_HEADER
                c.success_text = DELETE_USERGROUP_SUCCESS_TEXT
                c.url_ok       = "/administration/overviewUserGroups"
                return render('/administration/dialogs/success_deletegroup_user.mako')
            else:
                c.context     = "../main.mako"
                c.confirm_for = DELETE_USERGROUP_CONFIRM
                c.question    = DELETE_USERGROUP_CONFIRM_QUESTION
                c.url_yes     = "/administration/deleteUserGroup/%s/1" % ugo.id
                c.url_no      = "/administration/overviewUserGroups/"
                return render('/administration/dialogs/confirm_deleteusergroup.mako')
        except:
            return render('tests/trace.mako')
            c.failed_for  = DELETE_USERGROUP_FAILED_HEADER
            c.failed_text = DELETE_USERGROUP_FAILED_TEXT
            c.url_ok      = "/administration/overviewUserGroups"
            return render('/administration/dialogs/failed_new_usergroup.mako')

    @checkRole('admin_ka')
    def deleteUser(self, id, confirmed='0'):
        id = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        try:
            uo = UserObject(id)
            # Make sure that the user can not delete himself
            if uo.id == session.get('USER_AUTHORIZED').id:
                c.failed_for = DELETE_SELFUSER_FAILURE 
                c.failed_text  = DELETE_SELFUSER_FAILURE_TEXT
                c.url_ok     = "/administration/overviewUser"
                return render('/administration/dialogs/failed_delete_user.mako')

            if confirmed:
                # Check if there are cases for which the user is the editor. If
                # he is an editor then raise a dialog which allows the user to
                # transfer the cases.
                overview = CaseOverview()
                overview.search('editor:%s;state:1;state:2;state:3;state:4;state:5' % uo.id)
                if overview.getDatasets():
                    list   = UserListObject()
                    filter = [u.id for u in list.getAdminList()]
                    filter.append(uo.id) #the user we want to delete
                    c.user_list = list.getUserList(filter)
                    c.cases = overview
                    c.delete_user_id = id
                    return render('/administration/delete_user_helper.mako')
                else:
                    uo.delete()
                    c.notification_for = DELETE_USER_NOTIFICATION_SUCCESS 
                    c.notification_text = DELETE_USER_NOTIFICATION_TEXT_SUCCESS
                    c.success_for = DELETE_USER_SUCCESS_HEADER 
                    c.success_text = DELETE_USER_SUCCESS_TEXT
                    c.url_ok = "/administration/overviewUser"
                    return render('/administration/dialogs/success_delete_user.mako')

            else:
                c.context = "../main.mako" 
                c.confirm_for = DELETE_USER_CONFIRM 
                c.question = DELETE_USER_CONFIRM_QUESTION % (uo.last_name, uo.first_name, uo.login)
                c.url_yes = "/administration/deleteUser/%s/1" % id
                c.url_no  = "/administration/overviewUser/"
                return render('/administration/dialogs/confirm_deleteuser.mako')
        except:
            return render('/tests/trace.mako')

    @checkRole('admin_ka')
    def deleteUserHelper(self):
        validator           = DeleteUserHelperForm()
        all_cases           = CaseOverview()
        c.notification_for  = DIALOG_CONFIRM_SETEDITOR 
        c.form_errors       = {}
        c.form_result       = {}
        try:
            data        = formencode.variabledecode.variable_decode(request.params)
            form_result = validator.to_python(data)
            case_bundle = CaseBundle(form_result.get('case_id', []))
            new_editor  = UserObject(form_result.get('editor'))
            del_user    = UserObject(form_result.get('user_id'))
            all_cases.search('editor:%s;state:1;state:2;state:3;state:4;state:5' % del_user.id)
            num         = case_bundle.setEditor(form_result.get('editor'))
            rest        = all_cases.numDatasets() - case_bundle.numDatasets() 
            c.url_ok    = "/administration/deleteUser/%s/1" % form_result.get('user_id') 
            c.success_for = SETEDITOR_NOTIFICATION_SUCCESS_HEAD
            if rest > 0:
                c.success_for = SETEDITOR_NOTIFICATION_SUCCESS_REST_HEAD
                c.success_text = SETEDITOR_NOTIFICATION_SUCCESS_REST % (new_editor.first_name, new_editor.last_name, del_user.first_name, del_user.last_name)
            else:
                c.success_for = SETEDITOR_NOTIFICATION_SUCCESS_HEAD
                c.success_text = SETEDITOR_NOTIFICATION_SUCCESS % (new_editor.first_name, new_editor.last_name, del_user.first_name, del_user.last_name)
            return render('/administration/dialogs/notificate_delete_user_helper.mako')
        except formencode.Invalid, error:
            return self.deleteUser(form_result.get('user_id'), '1')
            

    @checkRole('admin_ka')
    def showSettings(self):
        c.form_errors = {}
        c.form_result = {}
        try:
            c.agency = Agency()
            return render('administration/show_settings.mako')
        except:
            return render('/tests/trace.mako')

    @checkRole('admin_ka')
    def editSettings(self):
        c.form_errors = {}
        c.form_result = {}
        try:
            c.agency = Agency()
            return render('administration/edit_settings.mako')
        except:
            return render('/tests/trace.mako')

    @checkRole('admin_ka')
    def editSettingsAction(self):
        validator = EditSettingsForm()
        c.agency        = Agency()
        try:
            c.form_result = validator.to_python(request.params)
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            return render('/administration/edit_settings.mako')

        c.agency.store(c.form_result)
        c.success_for   = EDIT_AGENCY_SUCCESS_HEADER 
        c.success_text  = EDIT_AGENCY_SUCCESS_TEXT
        c.url_ok        = "/administration/showSettings"
        return render ('/administration/dialogs/success_edit_agency.mako')
        #return self.showSettings()

    @checkRole('admin_ka')
    def viewPrivacyStatement(self):
        agency = Agency()
        statement = agency.getPrivacyStatement() 
        return statement

    @checkRole('admin_ka')
    def viewAidPlanStatement(self):
        agency = Agency()
        statement = agency.getAidPlanStatement() 
        return statement

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
