# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
#
import logging
import paste
import sys
import traceback

from waskaweb.lib.base import *

from waskaweb.model.document import \
    Document, \
    listDocuments, \
    deleteDocument

from pylons import config

from waskaweb.lib.security import checkRole

log = logging.getLogger(__name__)


SET_DOCUMENT_SUCCESS_HEADER         = u"""Dokument hinzugefügt!"""
SET_DOCUMENT_SUCCESS_TEXT           = u"""Bitte klicken Sie auf "OK", um fortzufahren."""
SET_DOCUMENT_FAILED_HEADER          = u"""Kein Dokument hinzugefügt!"""
SET_DOCUMENT_FAILED_TEXT            = u"""Vergewissern Sie sich, ob der angegebene Pfad korrekt ist. Dokumente können direkt über den "Browse" bzw. "Durchsuchen" Button ausgewählt werden. Bitte klicken Sie auf "OK", um fortzufahren. """
SET_DOCUMENT_SUCCESS_DELETE_HEADER  = u"""Dokument gelöscht! """
SET_DOCUMENT_SUCCESS_DELETE_TEXT    = u"""Bitte klicken Sie auf "OK", um fortzufahren."""
SET_DOCUMENT_CONFIRM_DELETE_HEADER  = u"""Dokument löschen?"""
SET_DOCUMENT_CONFIRM_DELETE_TEXT    = u"""Wollen Sie das Dokument wirklich löschen und die Daten unwiederbringlich verlieren?"""

SET_ATTACHMENT_SUCCESS_HEADER       = u"""Anlage hinzugefügt!"""
SET_ATTACHMENT_SUCCESS_TEXT         = u"""Bitte klicken Sie auf "OK", um fortzufahren."""
SET_ATTACHMENT_FAILED_HEADER        = u"""Keine Anlage hinzugefügt!"""
SET_ATTACHMENT_FAILED_TEXT          = u"""Vergewissern Sie sich, ob der angegebene Pfad korrekt ist. Anlagen  können direkt über den "Browse" bzw. "Durchsuchen" Button ausgewählt werden. Bitte klicken Sie auf "OK", um fortzufahren."""
SET_ATTACHMENT_SUCCESS_DELETE_HEADER    = u"""Anlage gelöscht!"""
SET_ATTACHMENT_SUCCESS_DELETE_TEXT      = u"""Bitte klicken Sie auf "OK", um fortzufahren."""
SET_ATTACHMENT_CONFIRM_DELETE_HEADER    = u"""Anlage löschen?"""
SET_ATTACHMENT_CONFIRM_DELETE_TEXT      = u"""Wollen Sie die Anlage wirklich löschen und die Daten unwiederbringlich verlieren?"""



class DocumentController(BaseController):

    @checkRole(('admin_ka', 'cm_ka'))
    def index(self):
        redirect_to(action='globalOverview')

    @checkRole(('admin_ka', 'cm_ka'))
    def globalOverview(self):
        try:
            c.files = listDocuments()
        except:
            traceback.print_exc(file=sys.stderr)
            c.files = []
            #return render('/tests/trace.mako')
        return render('/documents/global_overview.mako')

    @checkRole('admin_ka')
    def globalUpload(self):
        return render('/documents/global_upload.mako')

    @checkRole('admin_ka')
    def globalUploadAction(self):
        try:
            doc = Document()
            myfile = None
            try:
                myfile = request.POST.get('file')
                if not myfile is None:
                    doc.create(myfile.filename, myfile.file)
                    c.success_for = SET_DOCUMENT_SUCCESS_HEADER
                    c.success_text = SET_DOCUMENT_SUCCESS_TEXT
                    c.url_ok = h.url_for(controller="document", action="globalOverview")
                    return render('/documents/dialogs/success_documents.mako')
            finally:
                if not myfile is None:
                    try: myfile.file.close()
                    except: pass
        except:
            traceback.print_exc(file=sys.stderr)
            c.failed_for = SET_DOCUMENT_FAILED_HEADER
            c.failed_text = SET_DOCUMENT_FAILED_TEXT
            c.url_ok = h.url_for(controller="document", action="globalUpload")
            return render('/documents/dialogs/failed_document.mako')

            return self.globalUpload()


    @checkRole('cm_ka')
    def caseUploadAction(self):
        try:
            case = int(request.POST['case'])
        except:
            redirect_to(controller="case", action="documents") 

        try:
            doc = Document()
            myfile = None
            try:
                myfile = request.POST.get('file')
                if not myfile is None:
                    doc.create(myfile.filename, myfile.file, case)
                    c.success_for = SET_ATTACHMENT_SUCCESS_HEADER
                    c.success_text = SET_ATTACHMENT_SUCCESS_TEXT
                    c.url_ok = h.url_for(controller="document", action="caseUploadAction")
                    return render('/documents/dialogs/success_attachment.mako')
            finally:
                if not myfile is None:
                    try: myfile.file.close()
                    except: pass
        except:
            traceback.print_exc(file=sys.stderr)
            c.failed_for = SET_ATTACHMENT_FAILED_HEADER
            c.failed_text = SET_ATTACHMENT_FAILED_TEXT
            c.url_ok = h.url_for(controller="case", action="newDocument", id=case)
            return render('/documents/dialogs/failed_attachment.mako')
 

    @checkRole(('admin_ka', 'cm_ka'))
    def globalShow(self, id):
        id = self._checkInt(id)
        try:
            document = Document(id)
            document.load()
            data = paste.fileapp.DataApp(
                document.data,
                content_type=document.mime,
                cache_control='must-revalidate',
                pragma='public')
            return data(request.environ, self.start_response)
        except:
            traceback.print_exc(file=sys.stderr)
            redirect_to(controller="case", action="documents", id=case) 
            #return render('/tests/trace.mako')

    @checkRole(('admin_ka', 'cm_ka'))
    def caseShow(self, id):
        id = self._checkInt(id)
        try:
            document = Document(id)
            document.load(True)
            data = paste.fileapp.DataApp(
                document.data,
                content_type=document.mime,
                cache_control='must-revalidate',
                pragma='public')
            return data(request.environ, self.start_response)
        except:
            traceback.print_exc(file=sys.stderr)
            redirect_to(controller="case", action="documents") 
            #return render('/tests/trace.mako')

    @checkRole('admin_ka')
    def globalDelete(self, id, confirmed='0'):
        id        = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        if confirmed == 1:
            try:
                deleteDocument(id)
            except:
                traceback.print_exc(file=sys.stderr)
                return self.globalOverview()
                #return render('/tests/trace.mako')
            c.url_ok        =   h.url_for(controller="/document", action="globalOverview")
            c.success_for   =   SET_DOCUMENT_SUCCESS_DELETE_HEADER
            c.success_text  =   SET_DOCUMENT_SUCCESS_DELETE_TEXT
            return render('/documents/dialogs/delete_document_success.mako')
        else:
            c.context     = "../main.mako" 
            c.confirm_for = SET_DOCUMENT_CONFIRM_DELETE_HEADER
            c.question    = SET_DOCUMENT_CONFIRM_DELETE_TEXT
            c.url_yes     = "/document/globalDelete/%d/1" % id
            c.url_no      = "/document/globalOverview/"
            return render('/documents/dialogs/confirm.mako')

    @checkRole('cm_ka')
    def caseDelete(self, id, case, confirmed):
        id        = self._checkInt(id)
        case      = self._checkInt(case)
        confirmed = self._checkBool(confirmed)
        if confirmed == 1:
            try:
                deleteDocument(id, True)
            except:
                traceback.print_exc(file=sys.stderr)
                #return render('/tests/trace.mako') 
            c.url_ok        =   h.url_for(controller="/case", action="documents" , id=case)
            c.success_for   =   SET_ATTACHMENT_SUCCESS_DELETE_HEADER
            c.success_text  =   SET_ATTACHMENT_SUCCESS_DELETE_TEXT
            return render('/casemanagement/dialogs/delete_attachment_success.mako', id=case)
            #redirect_to(controller="/case", action="documents", id=case) 
        else:
            c.context     = "../main.mako" 
            c.confirm_for = SET_ATTACHMENT_CONFIRM_DELETE_HEADER
            c.question    = SET_ATTACHMENT_CONFIRM_DELETE_TEXT
            c.url_yes     = "/document/caseDelete/%d/%d/1" % (id, case)
            c.url_no      = h.url_for(controller="/case", action="documents", id=case)
            return render('/casemanagement/dialogs/confirm_delete_attachment.mako')

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
