# -*- coding: utf-8 -*-
#
# (c) 2008 by Intevation GmbH
# This is Free software under the GPLv3. 
# See LICENSE comming with the source of 'mpuls'
# for details.
#
# author: Torsten Irländer <torsten.irlaender@intevation.de>
#
import re, logging
from datetime import datetime
from waskaweb.lib.db import db
from waskaweb.lib.evaluation_new import *
log = logging.getLogger(__name__)

#JSA.02.12345.08
EXP_DOKNR  = re.compile('^JSA\.0[0-2]{1}\.([0-9]{5})\.[0-9]{2}$')
EXP_TYPE   = re.compile('^([0-9]{1,2},?)+$')
EXP_DATE   = re.compile('^\d{4}-\d{2}-\d{2}$')

SQL_TABLE      = "master_tbl_eval_total_view"
SQL_GENDER_FIELD   = 'geschlecht'
SQL_FKZ = """(TRUE)"""
SQL_CM = """
-- Nur CM-Faelle
((kompetenzfeststellung = '1' 
    or (kompetenzfeststellung = '-1' and phase in (2,3,4,5))
) 
or (bereich_berufliche_qualifizierung = '1' or bereich_berufsvorbereitung = '1'
   or bildender_bereich = '1' or bereich_lebensbewaeltigung = '1' or
   sonstiger_foerderbedarf = '1'))"""

# begin,ende,begin,begin,begin,ende,begin,ende,ende,begin,begin,begin,ende,begin,ende
SQL_TIME = """
(
    --
    -- Ausfiltern der Faelle, die definitiv nicht in Frage kommen
    --
    -- Status schwebend geloescht
    not (s.status = 3)
    -- Status nicht beendet aber schon anonymisiert
    and not (s.status = 5 and beendigung_5 is NULL)
    -- Beendigung vor BZ
    and not (coalesce(beendigung_5, now()) < '%s'::date)
    -- Begin nach BZ
    and not (coalesce(datum_feststellung, '1970-01-01'::date) > '%s'::date)
    -- Begin vor BZ aber auch letzte Aktualisierung vor BZ
    and not (coalesce(datum_feststellung, '1970-01-01'::date) < '%s'::date
             and s.zugriff < '%s'::date)
    -- Kein Begin und Erstgespraech nicht im BZ
    and not (datum_feststellung IS NULL and (coalesce(erstgespraech,
    '1970-01-01'::date) < '%s'::date or coalesce(erstgespraech,
    '1970-01-01'::date) > '%s'::date)
    )
)
and (
    --
    -- Nur Faelle die den Kriterien entsprechen 
    --
    -- Beendigung im BZ
    (coalesce(beendigung_5, now()) >= '%s'::date 
    and coalesce(beendigung_5, now()) <= '%s'::date) 
    or (
      -- Beendigung nach BZ
      coalesce(beendigung_5, now()) >= '%s'::date 
      and (
    -- Begin vor BZ und Aktualisierung nach Begin BZ
    (datum_feststellung < '%s'::date and s.zugriff >=
    '%s'::date) 
    or 
    -- Begin im BZ
    (datum_feststellung >= '%s'::date 
    and datum_feststellung <= '%s'::date)
      )
      -- Erstgespraech innerhalb des BZ
      or ( erstgespraech >= '%s'::date and erstgespraech <=
      '%s'::date)
    )
)
"""
# Alte Abfrage
#SQL_TIME = """(datum_feststellung <= '%s'::date and coalesce(beendigung_5, now()) >= '%s'::date)"""
# Für die erste Auswertung geändert. Auch alle Bestandskunden sollen zunächst
# als Eintritte gezählt werden.Damit ist die Anzahl der CM Kunden = Eintritte.
# SQL_EINTRITTE_GESAMT           = """('%s'::date <= datum_feststellung and '%s'::date >= datum_feststellung)"""
SQL_CMKUNDEN                   = """TRUE"""
SQL_EINTRITTE_GESAMT           = SQL_CMKUNDEN
SQL_AUSTRITTE_GESAMT           = """('%s'::date <= beendigung_5 and '%s'::date >= beendigung_5)"""
SQL_AUSTRITTE_VORZEITIG        = """(art_beendigung = 2 and '%s'::date <= beendigung_5 and '%s'::date >= beendigung_5)"""
SQL_AUSTRITTE_ABBRUCH          = """(art_beendigung = 1 and '%s'::date <= beendigung_5 and '%s'::date >= beendigung_5)"""
SQL_ALTER_15_24                = """(date_part('year', age(geburtsdatum)) >= 14 and date_part('year', age(geburtsdatum)) <= 24)"""
SQL_ALTER                      = """(date_part('year', age(geburtsdatum)) < 15 or date_part('year', age(geburtsdatum)) >= 25 and date_part('year', age(geburtsdatum)) <= 54 or date_part('year', age(geburtsdatum)) > 64)"""
SQL_ALTER_55_64                = """(date_part('year', age(geburtsdatum)) >= 55 and date_part('year', age(geburtsdatum)) <= 64)"""
SQL_ERWERBSTAETIG              = """(erwerbstaetigkeit_job = 1)"""
SQL_SELBSTSTAENDIG             = """(bei_eintritt_ist_der_jugendliche = 6)"""
SQL_ARBEITSLOSIGKEIT           = """(arbeitslosigkeit_1 = 1 and (bezug_alg2 = 1 and bezug_alg1 <> 1 or bezug_alg1 = 1 and bezug_alg2 <> 1))"""
SQL_ARBEITSLOSIGKEIT_LANG      = """(arbeitslosigkeit_1 = 1 and (bezug_alg2 = 1 and bezug_alg1 <> 1 or bezug_alg1 = 1 and bezug_alg2 <> 1) and dauer_nach_letzter_beschaeftigung > 12)"""
SQL_ARBEITSLOSIGKEIT_SGBII     = """(arbeitslosigkeit_1 = 1 and bezug_alg2 = 1 and bezug_alg1 <> 1)"""
SQL_ARBEITSLOSIGKEIT_SGBIII    = """(arbeitslosigkeit_1 = 1 and bezug_alg1 = 1 and bezug_alg2 <> 1)"""
SQL_NICHTERWERBSTAETIG         = """(erwerbstaetigkeit_job = 0 and arbeitslosigkeit_1 = 0)"""
SQL_NICHTERWERBSTAETIG_SCHULE  = """(erwerbstaetigkeit_job = 0 and arbeitslosigkeit_1 = 0 and bei_eintritt_ist_der_jugendliche = 1)"""
SQL_MIGRATIONSHINTERGRUND      = """(muttersprache not in (18,-1) or staatsangehoerigkeit not in (29,-1) or vater not in (1, -1) or mutter not in (1,-1))"""
SQL_BEHINDERUNG                = """(anerkannte_behinderung = 1)"""
SQL_SONST_BEEINTRAECHTIGUNG    = """(physische_beeintraechtigungen = 1 or psychische_beeintraechtigungen = 1)"""
SQL_OHNEABSCHLUSS              = """(schulabschluss_1 = 1)"""
SQL_ISCED1_2                   = """(schulabschluss_1 in (3,4,5,9,10))"""
SQL_ISCED3                     = """(schulabschluss_1 in (6))"""
SQL_ISCED4                     = """(schulabschluss_1 in (8))"""
SQL_AUSBLIDBUNG                = """(bei_eintritt_ist_der_jugendliche = 1)"""

EVALUATION = {'1': SQL_EINTRITTE_GESAMT,
              '2': SQL_AUSTRITTE_GESAMT,
              '3': SQL_AUSTRITTE_VORZEITIG,
              '4': SQL_AUSTRITTE_ABBRUCH, 
              '7': SQL_ALTER_15_24, 
              '8': SQL_ALTER, 
              '9': SQL_ALTER_55_64, 
              '10': SQL_ERWERBSTAETIG,
              '11': SQL_SELBSTSTAENDIG,
              '12': SQL_ARBEITSLOSIGKEIT,
              '13': SQL_ARBEITSLOSIGKEIT_LANG,
              '14': SQL_ARBEITSLOSIGKEIT_SGBII,
              '15': SQL_ARBEITSLOSIGKEIT_SGBIII,
              '16': SQL_NICHTERWERBSTAETIG,
              '17': SQL_NICHTERWERBSTAETIG_SCHULE,
              '19': SQL_MIGRATIONSHINTERGRUND,
              '20': SQL_BEHINDERUNG,
              '21': SQL_SONST_BEEINTRAECHTIGUNG,
              '22': SQL_OHNEABSCHLUSS,
              '23': SQL_ISCED1_2,
              '24': SQL_ISCED3,
              '25': SQL_ISCED4,
              '27': SQL_AUSBLIDBUNG,
              '28': SQL_CMKUNDEN
              }

DEFAULT_EVALUATIONS = ['1','2','3','4','7','8','12','13','14','15','16','17','19','22','23','28']

EVAL_NAMES = {'1': u'Eintritte gesamt',
              '2': u'Austritte gesamt',
              '3': u'Vorzeitig ausgetreten',
              '4': u'Abbrecher/innen', 
              '7': u'Jüngere Alterklassen (15-24)', 
              '8': u'Andere Altersklassen (<15, 25-54, >64)',
              '9': u'Ältere Alterklassen (55-64)', 
              '10': u'Erwerbstätige',
              '11': u'Selbstständige',
              '12': u'Gemeldete Arbeitslose',
              '13': u'Langzeitarbeitslose',
              '14': u'Arbeitslose SGBII-Bezug',
              '15': u'Arbeitslose SGBIII-Bezug',
              '16': u'Nicht-Erwerbspersonen',
              '17': u'Nicht-Erwerbspersonen in Ausbildung',
              '19': u'Menschen mit Migrationshintergrund',
              '20': u'Menschen mit Behinderung',
              '21': u'Menschen mit sonstiger Benachteiligung',
              '22': u'Ohne Schulabschluss',
              '23': u'ISCED 1, ISCED 2 (Nur Hauptschule/Realschule/BGJ/BVJ)',
              '24': u'ISCED 3 (Hochschulzugangsberechtigt 1. Bildungsweg)',
              '25': u'ISCED 4 (Hochschulzugangsberechtigt 2. Bildungsweg)',
              '27': u'Auszubildender/Auszubildende im Betrieb',
              '28': u'Gesamt CM-Kunden' 
              }

EVAL_DESCRIPTIONS = {'1': u'Anzahl der Teilnehmer/innen, die im Auswertungszeitraum neu in das CM aufgenommen wurden (Datum der Entscheidung zur Übernahme in das CM liegt innerhalb des Auswertungszeitraums).',
              '2': u'Gesamtanzahl der Teilnehmer/innen, die im Auswertungszeitraum das CM beendet haben.',
              '3': u'Anzahl der Teilnehmer/innen, die im Auswertungszeitraum das CM mit der Art "vorzeitige Beendigung" beendet haben.',
              '4': u'Anzahl der Teilnehmer/innen, die im Auswertungszeitraum das CM mit der Art "Abbruch" beendet haben.', 
              '7': u'Anzahl der Teilnehmer/innen in der Altersklasse 15-24 Jahre.', 
              '8': u'Anzahl der Teilnehmer/innen in den Altersklassen <15, 25-54, >64 Jahre.', 
              '9': u'Anzahl der Teilnehmer/innen in der Altersklasse 55-64 Jahre.', 
              '10': u'Anzahl der Teilnehmer/innen, die erwerbstätig sind (Biographische Daten).',
              '11': u'Anzahl der Teilnehmer/innen, die selbständig sind (Biographische Daten).',
              '12': u'Anzahl der Teilnehmer/innen, die als arbeitslos gemeldet sind (Biographische Daten).',
              '13': u'Anzahl der Teilnehmer/innen, die länger als 12 Monate arbeitslos gemeldet sind.',
              '14': u'Anzahl der Teilnehmer/innen, die Unterstützunsleistungen nach SGBII beziehen.',
              '15': u'Anzahl der Teilnehmer/innen, die Unterstützunsleistungen nach SGBIII beziehen.',
              '16': u'Anzahl der Teilnehmer/innen, die weder erwerbstätig noch arbeitslos gemeldet sind.',
              '17': u'Anzahl der nicht erwerbstätigen Teilnehmer/innen, die sich bei Eintritt in einer Ausbildung befanden (Zuständigkeiten, Eingansdaten).',
              '19': u'Anzahl der Teilnehmer/innen mit Migrationshintergrund.',
              '20': u'Anzahl der Teilnehmer/innen mit annerkannter Behinderung.',
              '21': u'Anzahl der Teilnehmer/innen mit sonstiger Beeinträchtigung (physische oder psychische).',
              '22': u'Anzahl der Teilnehmer/innen ohne Schulabschluss.',
              '23': u'Anzahl der Teilnehmer/innen, die einen Schulabschluss nach dem internationalen ISCED 1 oder ISCED 2 Standard haben.',
              '24': u'Anzahl der Teilnehmer/innen, die einen Schulabschluss nach dem internationalen ISCED 3 Standard haben.',
              '25': u'Anzahl der Teilnehmer/innen, die einen Schulabschluss nach dem internationalen ISCED 4 Standard haben.',
              '27':  u'Anzahl der Teilnehmer/innen, die sich beim Zugang in der Ausbildung befunden haben (Zuständigkeiten, Eingansdaten).',
              '28': u'Anzahl der Teilnehmer/innen, die sich während des Auswertungszeitraums im CM befunden haben.' 
              }

class AdeleXML(Evaluation):
    def __init__(self, params):
        Evaluation.__init__(self, params)

    def _setup(self):
        self.evaluation_mapping  = EVALUATION
        self.default_evaluations = DEFAULT_EVALUATIONS
        self.sql_base_query      = SQL_BASE_QUERY
        self.sql_table           = SQL_TABLE
        self.sql_gender          = SQL_GENDER_FIELD

        #custom
        self.sql_time_frame      = SQL_TIME
        self.sql_fkz             = SQL_FKZ
        self.sql_cm              = SQL_CM

    def _build_sql(self, where):
        out = []
        try:
            #out.append(self.sql_fkz % self.agency)
            out.append(self.sql_cm)
            # begin,ende,begin,begin,begin,ende
            begin = self.start_date
            ende  = self.end_date
            out.append(self.sql_time_frame % (begin,ende,begin,begin,begin,ende,begin,ende,ende,begin,begin,begin,ende,begin,ende))

        except StandardError, err:
            print "Fehler: %s" % err
        try:
            out.append(where % (self.start_date, self.end_date))
        except StandardError, err:
            out.append(where)
            pass # Maybe the where clause does not have any time restrictions
        where = " and ".join(out)
        sql = Evaluation._build_sql(self, where)
        return sql

#class AdeleXML:
#
#    def perform(self, params):
#        doknr      = params.get('doknr')
#        start_date = params.get('start_date')
#        end_date   = params.get('end_date')
#        typelist   = params.get('typelist')
#
#        #TEST Remove for production
#        doknr = 'JSA.02.12345.08'
#        start_date = '2008-01-01'
#        end_date = '2008-12-31'
#
#        #Do some checks on the incoming data
#        if not doknr or not EXP_DOKNR.match(doknr): return "INVALID: 'doknr'"
#        if not start_date or not EXP_DATE.match(start_date): return "INVALID: 'start_date'"
#        if not end_date or not EXP_DATE.match(end_date): return "INVALID: 'end_date'"
#        if not typelist:
#            TYPELIST = DEFAULT_EVALUATIONS
#        else:
#            if not EXP_TYPE.match(typelist): return "INVALID: 'typelist'"
#            TYPELIST = typelist.split(',')
#
#        #Make evaluation on the collected cases
#        try:
#            out = []
#            out.append(XML_HEADER % (doknr, start_date, end_date)) 
#            for id in TYPELIST:
#                SQL_WHERE = EVALUATION.get(str(id), 'TRUE')
#
#                # If the timeinterval was not or not full provied set timeframe
#                # to a very large interval 
#                if not start_date or not end_date:
#                    start_date = '1970-01-01'
#                    end_date = '%s' % datetime.today().strftime('%Y-%m-%d') 
#
#                TIMEFRAME = SQL_TIME % (end_date, start_date)
#
#                # Try to set the dates in the where clause
#                try:
#                    SQL_WHERE = SQL_WHERE % (start_date, end_date)
#                except:
#                    pass # Ignore. Maybe the sql query does not have any
#                         # datefields
#
#                sql = SQL_BASE_QUERY % (SQL_GROUPBY, SQL_TABLE, TIMEFRAME, SQL_WHERE , SQL_GROUPBY)
#                eval = EvaluationQuery(id, sql)
#                eval.perform()
#                out.append(XML_EVALBLOCK % (eval.id, eval.men, eval.women, eval.men+eval.women))
#            out.append(XML_FOOTER)
#        except StandardError, err:
#            log.error(err)
#
#        return "\n".join(out)

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
