# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
"""The application's Globals object"""
import pylons
from pylons import config

from waskaweb.model.io.document import openDocument 
import waskaweb.model.data as data 
from waskaweb.lib.formular_help import HelpProvider 

import waskaweb.lib.security    as security
from  waskaweb.lib.helpers import load_db_list_from_file,DBS_LIST

from waskaweb.lib.timelog import setupTimeLogging
from waskaweb.lib.config import MpulsConfig, get_path

import os, sys, traceback

class Globals(object):
    """Globals acts as a container for objects available throughout the
    life of the application
    """

    def __init__(self):
        """One instance of Globals is created during application
        initialization and is available during requests via the 'g'
        variable
        """
        # Load mpuls configuration
        config_file = pylons.config['app_conf'].get('mpuls_config', 'mpuls.json')
        if config_file:
            if not os.path.isabs(config_file):
                root = pylons.config.get('pylons.paths').get('root')
                config_file = os.path.join(root, '..', config_file)


        self.config_path = os.path.dirname(config_file)
        self.mpuls_config = MpulsConfig(config_file)

        # XXX: Dead ugly!
        path = os.path.join(os.path.dirname(__file__), '..', '..', 'formed')
        treeFile  = os.path.join(path, 'formedtree_web.xml')
        helpFile  = os.path.join(path, 'waska-hilfetexte.xhtml')
        annonFile = os.path.join(path, 'annotations.xhtml')

        # XXX: Check why the following throws key exceptions. Is config var not
        # availiable in Globals?!?
        #treeFile = os.path.join(os.path.dirname(__file__), config['formed_tree'])
        self.formedTree = openDocument(treeFile)
        try:
            f = open(helpFile, "rb")
            help_xhtml = f.read()
            f.close()
            f = open(annonFile)
            annon_xhtml = f.read()
            f.close()
            self.helpData = HelpProvider(help_xhtml, annon_xhtml) 
        except:
            traceback.print_exc(file=sys.stderr)
            print >> sys.stderr, "Could not open Helpfile"
            self.helpData = None

        # prefetch external external choice leaves
        for ex in self.formedTree.findAllByClass(data.ExternalChoiceListLeaf):
            fname = ex.getValue()
            if not fname: continue
            fname = os.path.join(path, fname)
            subDoc = openDocument(fname)
            ex.choice = subDoc.findByClassAndName(data.ChoiceNode)

        # data for database connections

        host   = config.get('db_host')
        port   = config.get('db_port')
        dbname = config.get('db_database')
        user   = config.get('db_user')

        if not host   is None: security.HOST   = host
        if not port   is None: security.PORT   = int(port)
        if not dbname is None: security.DBNAME = u"%s" % dbname
        if not user   is None: security.USER   = u"%s" % user

        # set time logging
        time_host = config.get('time_host')
        time_port = config.get('time_port')

        if time_host and time_port:
            setupTimeLogging(time_host, int(time_port))
        elif time_host:
            setupTimeLogging(int(time_host))

        # File to remap database name
        mapping_file = config.get('db_mapping_file')

        if mapping_file:
            security.load_db_mapping_from_file(mapping_file)

        # File to define section list (instead of reading from client cert)
        list_file = config.get('db_list_file')

        if list_file:
            load_db_list_from_file(list_file)

