# -*- coding: utf-8 -*-
#
# (c) 2008 by Intevation GmbH
# This is Free software under the GPLv3. 
# See LICENSE comming with the source of 'mpuls'
# for details.
#
# author: Torsten Irländer <torsten.irlaender@intevation.de>
#
import re, logging
from datetime import datetime
from waskaweb.lib.db import db
from waskaweb.lib.evaluation_new import *
log = logging.getLogger(__name__)

#JSA.02.12345.08
EXP_DOKNR  = re.compile('^JSA\.0[0-2]{1}\.([0-9]{5})\.[0-9]{2}$')
EXP_TYPE   = re.compile('^([0-9]{1,2},?)+$')
EXP_DATE   = re.compile('^\d{4}-\d{2}-\d{2}$')

SQL_TABLE      = "master_tbl_eval_total_view"
SQL_GENDER_FIELD   = 'geschlecht'
SQL_FKZ = """(TRUE)"""
# ende
SQL_CM = """
-- Nur CM-Faelle
-- Faelle, die vor dem Ende des BZ zum CM geworden sind 
( coalesce(datum_feststellung, now()) <= '%s'::date) 
and ((kompetenzfeststellung = '1' 
    or (kompetenzfeststellung = '-1' and phase in (2,3,4,5))
) 
or (bereich_berufliche_qualifizierung = '1' or bereich_berufsvorbereitung = '1'
   or bildender_bereich = '1' or bereich_lebensbewaeltigung = '1' or
   sonstiger_foerderbedarf = '1'))"""

# begin,ende,begin,begin
SQL_TIME2 = """TRUE"""
SQL_TIME = """
(
    --
    -- Ausfiltern der Faelle, die definitiv nicht in Frage kommen
    --
    -- Status schwebend geloescht
    not (s.status = 3)
    -- Status nicht beendet aber schon anonymisiert
    and not (s.status = 5 and beendigung_5 is NULL)
    -- Beendigung vor BZ
    and not (coalesce(beendigung_5, now()) < '%s'::date)
    -- Begin nach BZ
    and not (coalesce(erstgespraech, '1970-01-01'::date) > '%s'::date)
    -- Begin vor BZ aber auch letzte Aktualisierung vor BZ
    and not (coalesce(erstgespraech, '1970-01-01'::date) < '%s'::date
             and s.zugriff < '%s'::date)
)
"""
SQL_GESAMT = """TRUE"""
SQL_CMKUNDEN = SQL_CM

EVALUATION = {'1': SQL_GESAMT,
              '2': SQL_CMKUNDEN
              }

DEFAULT_EVALUATIONS = ['1','2']

EVAL_NAMES = {'1': u'Fälle gesamt',
              '2': u'CM-Fälle gesamt',
              }

EVAL_DESCRIPTIONS = {'1': u'Anzahl der Teilnehmer/innen, die im Auswertungszeitraum in der Einrichtung betreut wurden.',
              '2': u'Gesamtanzahl der Teilnehmer/innen, die im Auswertungszeitraum als CM-Kunden betreut wurden.',
              }

class BasicEvaluation(Evaluation):
    def __init__(self, params):
        Evaluation.__init__(self, params)

    def _setup(self):
        self.evaluation_mapping  = EVALUATION
        self.default_evaluations = DEFAULT_EVALUATIONS
        self.sql_base_query      = SQL_BASE_QUERY
        self.sql_table           = SQL_TABLE
        self.sql_gender          = SQL_GENDER_FIELD

        #custom
        self.sql_time_frame      = SQL_TIME
        self.sql_fkz             = SQL_FKZ

    def _build_sql(self, where):
        out = []
        try:
            begin = self.start_date
            ende  = self.end_date
            out.append(self.sql_time_frame % (begin,ende,begin,begin))
        except StandardError, err:
            print "Fehler: %s" % err
        try:
            out.append(where % (self.end_date))
        except StandardError, err:
            out.append(where)
            pass # Maybe the where clause does not have any time restrictions
        where = " and ".join(out)
        sql = Evaluation._build_sql(self, where)
        return sql

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
