# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
#

import sys
import re
from datetime import datetime
from threading import Lock
from waskaweb.lib.db import db
from waskaweb.model.phases_factory import RequiredFields
import psycopg2.extras
from waskaweb.lib.base import session, g, h, config 

SAVE_SEARCH = re.compile(r'[^\w:;\-\. ]', re.UNICODE)

VALID_OPTIONS = ['editor', 'own', 'standin', 'state', 'sort_field', 'sort_order', 'interval_start_date', 'interval_end_date', 'interval_start_field', 'interval_end_field', 'gender', 'phase', 'branch', 'fkz', 'inconsistency', 'only_cases_after', 'bad', 'field']

SEARCH_CASE_SQL = \
""" SELECT
      m.id,
      m.phase,
      m.name_1,
      m.vorname_1,
      m.kundennummer,
      m.bearbeiter_id,
      st.status,
      st.zugriff,
      st.id as s_id,
      m.zeitraum_nachbetreuung,
      m.art_beendigung,
      m.einverstaendniserklaerung
    FROM master_tbl_view m
    JOIN ka_status_tbl_view st ON
    m.id = st.master_id
    JOIN ka_benutzer_tbl b ON
    m.bearbeiter_id = b.id
    WHERE
      ((m.vorname_1 ~* %%(needle)s
      OR m.name_1 ~* %%(needle)s
      OR m.kundennummer ~* %%(needle)s)
      OR ( m.vorname_1 IS NULL
      AND m.name_1 IS NULL
      AND m.kundennummer IS NULL
      ))
      AND st.status IN (%s)
      AND %s
      AND %s
      AND %s
      AND %s
      AND %s
      AND %s
      AND %s
      AND %s
      AND %s
      AND %s
    ORDER BY %s %s"""

INCONSISTENCY_LOCK = None
INCONSISTENCY_TYPE_QUERY = None
INCONSISTENCY_CHECK_AFTER = '2008-08-31'

SEARCH_AGENCY_SQL = \
"""select count(m.id) as anzahl,
m.fkz,
date_trunc('minute', max(s.zugriff)) as zugriff
from master_tbl_view m 
join ka_status_tbl_view s 
on m.id = s.master_id
WHERE %s
AND %s
AND %s
group by fkz
ORDER BY %s %s"""


def build_inconsistency_type_querys():

    # TODO: 5,6 seems to be obsolete as this inconsistency will be covered by
    # the phases
    querys = {'1': 'm.erstgespraech is NULL', 
              '2': 'm.erstgespraech::timestamp > now()',
              '3': 'm.erstgespraech::timestamp > m.beendigung_5::timestamp',
              '4': 'm.beendigung_5::timestamp > now()',
              '5': '(m.art_beendigung <> -1 AND m.beendigung_5 is NULL)',
              '6': '(m.art_beendigung = -1 AND m.beendigung_5 is not NULL)',
              '7': '(m.phase = -1)'}

    #Now fetch more inconsistencys based on phases
    rf = RequiredFields()
    rf.extractRequiredFields(g.formedTree)
    sql = rf.getSQLWhereClauses()
    for s,q in sql:
        querys[s] = q 
    return querys

class Search:
    def __init__(self):
        self.sql = None

    def _parse_search_str(self, s):
        # TODO This is a very very simple approach. We should defentiley
        # implement some searchengine here
        search = []
        allowed_states = [-1]
        bad_types      = []
        phases         = []
        fields         = {}
        options        = {}

        if s:
            s = SAVE_SEARCH.sub(u'', s)
            option_list = [o.strip() for o in s.split(';')]
            for option in option_list:
                if option.find(":") > 0:
                    try:
                        key, value = [x.strip() for x in option.split(":")]
                        if key in VALID_OPTIONS:
                            if key == 'state':
                                allowed_states.append(int(value))
                            if key == 'phase':
                                phases.append(int(value))
                            #if key == 'field':
                            #    field, operator, fvalue = [x.strip() for x in value.split(" ")]
                            #    fields[field] = fvalue
                            if key in ['inconsistency', 'bad']:
                                bad_types.append(value)
                            else:
                                options[key] = value
                    except ValueError, err :
                        print >> sys.stderr, "Found invalid search syntax in options: %s" % err
                else:
                    search.append(option)
        return search, options, allowed_states, bad_types, phases, fields

    def get_sql(self):
        return self.sql

    def _build_sql(self, search, options, allowed_states, bad_types, phases, fieldsdic):
        pass

    def queryDB(self, sql, fields):
        '''Does the search on the DB and returns the result of the search'''
        rows = []
        con, cur = None, None
        try:
            try:
                con = db.getConnection()
                cur = con.cursor(cursor_factory=psycopg2.extras.DictCursor)
                cur.execute(sql, fields)
                # save sql 
                self.sql = cur.query
                rows = cur.fetchall()
            except:
                return {}
        finally:
            db.recycleConnection(con, cur)
        return rows

    def perform(self, search_str):
        '''Returns the result set of a search based on the search string'''
        search, options, states, bad_types, phases, fields = self._parse_search_str(search_str)
        sql, fields = self._build_sql(search, options, states, bad_types, phases, fields)
        result = self.queryDB(sql, fields)
        return result

class AgencySearch(Search):
    def __init__(self):
        Search.__init__(self)

    def _build_sql(self, search, options, allowed_states, bad_types, phaseslist, fieldsdic):
        fields = {'needle': ""}
        gender = options.get('gender') and "m.geschlecht = %s" % options.get('gender') or "TRUE"

        #Phases
        phases = 'TRUE' #default
        if phaseslist:
            phases = 'm.id in (SELECT id from master_tbl_view WHERE phase in (%s))' % ','.join([str(p) for p in phaseslist])

        time_interval = "TRUE"
        interval_start_field = options.get('interval_start_field', "erstgespraech")
        interval_end_field = options.get('interval_end_field', "beendigung_5")
        interval_start_date = options.get('interval_start_date',"1970-01-01")
        d = datetime.today()
        interval_end_date = options.get('interval_end_date', d.strftime('%Y-%m-%d'))
        fields = {'start_field': interval_start_field, 'start_date': interval_start_date, 'end_field': interval_end_field, 'end_date': interval_end_date} 
        if interval_start_date != '1970-01-01':
            time_interval = "(coalesce(%(start_field)s, '1970-01-01'::date) <= '%(end_date)s'::date \
            AND (coalesce(%(end_field)s, now()) >= '%(start_date)s'::date))" % fields

            #time_interval = "(coalesce(%(start_field)s, '1970-01-01'::date) <= '%(end_date)s'::date AND \
            #coalesce(%(start_field)s, '1970-01-01'::date) <= coalesce(%(end_field)s, st.zugriff) AND \
            #((m.id in (SELECT id from master_tbl_incm_view) AND \
            #(coalesce(%(end_field)s, now()) >= '%(start_date)s'::date)) OR \
            #(st.zugriff >= '%(start_date)s'::date)))" % fields

        sort_field = options.get('sort_field', "name_1")
        sort_order       = options.get('sort_order', "desc")
        fields['needle'] = " ".join(search) 
        SQL = SEARCH_AGENCY_SQL % (gender, time_interval, phases, sort_field, sort_order) 
        return (SQL, fields)

class CaseSearch(Search):
    def __init__(self):
        Search.__init__(self)

    def _build_sql(self, search, options, allowed_states, bad_types, phaseslist, fieldsdic):
        fields = {'needle': ""}
        # Show all cases regardless if the privacy statement has been declined
        #ee =  h.hasRole(['cm_ka']) and "m.einverstaendniserklaerung <> 0" or "TRUE"
        ee = "TRUE"

        which_user = 0
        allowed_viewer = "FALSE"
        if options.has_key('own'): which_user += 2
        if options.has_key('standin'): which_user += 1
        if which_user == 1: allowed_viewer = "m.bearbeiter_id <> %s" % options.get('standin')
        if which_user == 2: allowed_viewer = "m.bearbeiter_id = %s" % options.get('own')
        if which_user == 3: allowed_viewer = "TRUE"
        gender = options.get('gender') and "m.geschlecht = %s" % options.get('gender') or "TRUE"
        allowed_viewer =  h.hasRole(['admin_ka', 'pb_ka']) and "TRUE" or allowed_viewer
        editor = options.has_key('editor') and "m.bearbeiter_id = %s" % options.get('editor') or "TRUE"
        sort_field = options.get('sort_field', "name_1")
        sort_order       = options.get('sort_order', "desc")
        branch = options.get('branch') and "b.filiale = '%s'" % options.get('branch') or "TRUE"
        fkz = options.get('fkz') and "m.fkz = '%s'" % options.get('fkz') or "TRUE"

        #Identify bad cases
        bad_query = []

        global INCONSISTENCY_LOCK 
        INCONSISTENCY_LOCK= Lock()
        global INCONSISTENCY_TYPE_QUERY
        INCONSISTENCY_LOCK.acquire()
        try:
            try:
                if not INCONSISTENCY_TYPE_QUERY:
                    INCONSISTENCY_TYPE_QUERY = build_inconsistency_type_querys()
            except:
                INCONSISTENCY_TYPE_QUERY = {}
        finally:
            INCONSISTENCY_LOCK.release()

        for type_id in bad_types:
            query = INCONSISTENCY_TYPE_QUERY.get(str(type_id))
            if query:
                bad_query.append(query)
        if len(bad_query) > 0:
            bad = "(%s)" % " OR ".join(bad_query)
        else:
            bad = 'TRUE'

        # This is a convinience option to ignore older cases in the
        # inconsistency search
        if options.get('only_cases_after'):
            only_cases_after = "coalesce(erstgespraech, now()) > '%s'" % options.get('only_cases_after')
        else:
            only_cases_after = 'TRUE'

        #Build timeinterval
        time_interval = "TRUE"
        if options.get('interval_start_date') and options.get('interval_end_date'):
            interval_start_field = options.get('interval_start_field', "erstgespraech")
            interval_end_field = options.get('interval_end_field', "beendigung_5")
            interval_start_date = options.get('interval_start_date',"1970-01-01")
            d = datetime.today()
            interval_end_date = options.get('interval_end_date', d.strftime('%Y-%m-%d'))
            fields = {'start_field': interval_start_field, 'start_date': interval_start_date, 'end_field': interval_end_field, 'end_date': interval_end_date} 
            if interval_start_date != '1970-01-01':
                time_interval = "(coalesce(%(start_field)s, '1970-01-01'::date) <= '%(end_date)s'::date \
                AND (coalesce(%(end_field)s, now()) >= '%(start_date)s'::date))" % fields

                #time_interval = "(coalesce(%(start_field)s, '1970-01-01'::date) <= '%(end_date)s'::date AND \
                #coalesce(%(start_field)s, '1970-01-01'::date) <= coalesce(%(end_field)s, st.zugriff) AND \
                #((m.id in (SELECT id from master_tbl_incm_view) AND \
                #(coalesce(%(end_field)s, now()) >= '%(start_date)s'::date)) OR \
                #(st.zugriff >= '%(start_date)s'::date)))" % fields

        #Phases
        phases = 'TRUE' #default
        if phaseslist:
            phases = 'm.id in (SELECT id from master_tbl_view WHERE phase in (%s))' % ','.join([str(p) for p in phaseslist])

        ##User defined fieldsearch
        #fieldsearchlist = [] 
        #for field, value in fieldsdic.iteritems():
        #    fieldsearchlist.append("%s = '%s'" % field, value)
    
        #if fieldsearchlist:
        #    fieldsearch = "(%s)" % " AND ".join(fieldsearchlist)
        #else:
        #    fieldsearch = 'TRUE'

        fields['needle'] = " ".join(search) 
        SQL = SEARCH_CASE_SQL % (",".join([str(x) for x in allowed_states]), ee, allowed_viewer, editor, gender, time_interval, phases, branch, fkz, bad, only_cases_after, sort_field, sort_order) 
        return (SQL, fields)

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
