# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
#


import sys
import re
from datetime import date, datetime
import traceback
import psycopg2.extras
import codecs
import cStringIO as StringIO
import shutil
from locale import strcoll
from sets import Set
from pylons.i18n                import _, ungettext, N_
from waskaweb.lib.db import db
from waskaweb.lib.sql_helper import where_ids, range_compress 
from waskaweb.lib.search import CaseSearch
from waskaweb.lib.csv import exportAsCSV
from waskaweb.lib.excel import exportAsXLS
from waskaweb.lib.xmlexport import exportAsXML
from waskaweb.lib.base import session, g, h, config 
import waskaweb.lib.filters as F
from waskaweb.model.user import UserObject 
from waskaweb.model.agencysettings import Agency
from waskaweb.model.repeatgroup import AidList, CREATE_RG_BILDEN_DS,CREATE_RG_LEBEN_DS, CREATE_RG_QUALI_DS, CREATE_RG_BERUF_DS
from waskaweb.model.statement import PrivacyStatement, AidPlanStatement, DiscretionStatement
from waskaweb.model.document import listDocuments 

from waskaweb.model.phase_transition import phase_description, phase_end, \
                                            CLEAR_START, CLEAR_ENDE
from waskaweb.model.phases_factory   import RequiredFields, currentPhase, phases_pairs

from waskaweb.lib.helpers import dd_mm_yyyy_HH_MM, dd_mm_YYYY

from os import tmpnam, unlink

UNKNOWN_DATE = date(1,1,1)

def save_getone(params, k, v):
    try:
        return params.getone(k)
    except KeyError:
        return v

# SQL STATEMENTS
LOAD_CASESTATE_SQL = """
SELECT 
    s.id, 
    s.status, 
    s.zugriff, 
    m.zeitraum_nachbetreuung AS cm_beendet, 
    m.art_beendigung AS cm_abbruch, 
    getLastCaseAppointment(m.id) AS letzer_termin,
    m.phase AS phase
FROM ka_status_tbl_view s JOIN master_tbl_view m ON m.id = s.master_id 
WHERE s.master_id = %(id)s
"""

SET_CASESTATE_SQL = """
UPDATE ka_status_tbl_view 
    SET status = %(status)s 
    WHERE id = %(id)s"""

SET_CASEPHASE_SQL = """
    UPDATE master_tbl_view 
    SET phase = %(phase)s 
    WHERE id IN (SELECT master_id FROM ka_status_tbl_view WHERE id = %(id)s)"""

LOAD_CASE_DIGEST_SQL = """
SELECT
    name_1, 
    vorname_1, 
    kundennummer, 
    date_part('year', age(geburtsdatum::timestamp)) AS alter,
    plz, 
    strasse, 
    wohnort, 
    vorwahl, 
    telefon_1,
    mobiltelefon, 
    telefon_sonstige, 
    email, 
    einverstaendniserklaerung, 
    name_2, 
    vorname_2, 
    vorwahl_1, 
    telefon_1_1, 
    mobiltelefon_1, 
    email_1,
    art_beendigung,
    vermittlung
FROM master_tbl_view WHERE id = %(id)s
"""

LOAD_CASE_SQL = """SELECT name_1, vorname_1, bearbeiter_id, kundennummer, erstgespraech, beendigung_5, zugang_kompetenzagentur, letzte_jahr_allg_schule, prognose_liegt_vor from master_tbl_view WHERE id = %(id)s"""
DELETE_CASE_SQL = """select delete_master_ds(%(id)s)""" 

CREATE_CASE_SQL = """SELECT create_master_ds(%(uuid)s)"""
INIT_CASE_SQL= """UPDATE page_0_view SET name_kompetenzagentur = %(ka_name)s, fkz = %(fkz)s, vorname = %(vorname)s, name = %(nachname)s WHERE id = %(id)s"""

CREATE_RG_KOMPETENZ_DS = """SELECT create_rg_kompetenz_ds(%(id)s, %(uuid)s)"""

LOAD_STANDIN_SQL = """SELECT n.benutzer_id FROM nm_benutzer_master_tbl_view n
                        JOIN master_tbl_view m ON 
                        m.id = n.master_id
                        WHERE m.id = %(id)s"""
ADD_STANDIN_SQL = """SELECT create_nm_benutzer_master_ds(%(groupid)s, %(caseid)s)"""
DELETE_STANDIN_SQL = """SELECT delete_nm_benutzer_master_ds(%(groupid)s, %(caseid)s)"""
SET_EDITOR_SQL = """SELECT set_case_editor(%(userid)s, %(caseid)s)"""
SET_EDITOR_NAME_SQL = """SELECT set_case_editor_name(%(firstname)s, %(lastname)s, %(caseid)s)"""
FETCH_KOMPETENZ_ID = """SELECT id from rg_kompetenzfestellung_tbl_view WHERE master_id = %(id)s LIMIT 1"""

CASE_EXISTS = "SELECT * FROM case_exists(%(knummer)s)"

CREATE_SQL = {
   'rg_angebote_bildenden_bereich_tbl':         CREATE_RG_BILDEN_DS,
   'rg_angebote_lebensbewaeltigung_tbl':        CREATE_RG_LEBEN_DS,
   'rg_angebote_berufliche_qualifizierung_tbl': CREATE_RG_QUALI_DS,
   'rg_angebote_berufsvorbereitung_tbl':        CREATE_RG_BERUF_DS
}

ANONYMIZE_CASE          = """SELECT anonymize_case_wrapper(%(id)s)"""

LOAD_ART_BEENDIGUNG = """SELECT value from art_beendigung_tbl where id = %(id)s"""
LOAD_STATUS_BEENDIGUNG = """SELECT value from vermittlung_tbl where id = %(id)s"""

class LoadCaseNotExistsError(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)

class ImportCaseExistsError(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)

class CaseList:
    def __init__(self, case_ids=None):
        self.case_list = case_ids or []

    def listDatasetIds(self):
        return self.case_list

    def isEmpty(self):
        return len(self.case_list) == 0

    def numDatasets(self):
        return len(self.case_list) 

    def getCases(self, filter=None):
        list = []
        if filter:
            cases = [id for id in self.case_list if id not in filter]
        else:
            cases = self.case_list
        for id in cases:
            factory = CaseFactory()
            list.append(factory.loadById(id))
        return list

class CaseBundle(CaseList):
    def __init__(self, case_ids=None):
        CaseList.__init__(self, case_ids)

    def setEditor(self, user_id):
        factory = CaseFactory()
        num = 0
        for id in self.case_list:
            case = factory.loadById(id)
            try:
                case.setEditor(user_id)
                num += 1
            except:
                print >> sys.stderr, "Could not set editor for case in bundle"
        return num

    def setStandin(self, group_ids):
        factory = CaseFactory()
        num = 0
        for id in self.case_list:
            case = factory.loadById(id)
            try:
                standin = case.getStandin()
                standin.setGroups(group_ids)
                num += 1
            except:
                print >> sys.stderr, "Could not set standin for case in bundle"
        return num

    def delete(self):
        factory = CaseFactory()
        num = 0
        for id in self.case_list:
            case = factory.loadById(id)
            try:
                case.delete()
                num += 1
            except:
                print >> sys.stderr, "Could not delete Case from bundle"
        return num

    def anonymize(self):
        factory = CaseFactory()
        num = 0
        for id in self.case_list:
            case = factory.loadById(id)
            try:
                case.anonymize()
                num += 1
            except:
                print >> sys.stderr, "Could not anonymize Case from bundle"
        return num

    def restore(self):
        factory = CaseFactory()
        num = 0
        for id in self.case_list:
            case = factory.loadById(id)
            try:
                case.restore()
                num += 1
            except StandardError, e:
                print >> sys.stderr, "Could not restore case in bundle: %s" % e
        return num

    def markAnonymize(self):
        factory = CaseFactory()
        num = 0
        for id in self.case_list:
            case = factory.loadById(id)
            try:
                case.getState().setState(4)
                num += 1
            except:
                print >> sys.stderr, "Could not mark case for anonymisation in bundle"
        return num

    def markDelete(self):
        factory = CaseFactory()
        num = 0
        for id in self.case_list:
            case = factory.loadById(id)
            try:
                state = case.getState()
                state.setState(3)
                num += 1
            except:
                print >> sys.stderr, "Could not mark Case to delete from bundle"
        return num

    def exportAsCSV(self, case_parts=None, anonymize=False):
        con, cur = None, None
        f = StringIO.StringIO()
        Writer = codecs.getwriter('latin-1')
        out = Writer(f, errors='replace')
        try:
            con = db.getConnection()
            cur = con.cursor()
            exportAsCSV(cur, out, self.listDatasetIds(), case_parts, anonymize)
            out.flush()
            bytes = out.getvalue()
            out.close()
            return bytes
        finally:
            db.recycleConnection(con, cur)

    def exportAsXML(self, case_parts = None, anonymize=False):
        con, cur = None, None
        f = StringIO.StringIO()
        Writer = codecs.getwriter('utf-8')
        out = Writer(f, errors='replace')
        try:
            con = db.getConnection()
            cur = con.cursor(cursor_factory=psycopg2.extras.DictCursor)
            exportAsXML(cur, out, self.listDatasetIds(), case_parts, anonymize)
            out.flush()
            bytes = out.getvalue()
            out.close()
            return bytes
        finally:
            db.recycleConnection(con, cur)

    def exportAsXLS(self, case_parts = None, anonymize=False):
        con, cur = None, None
        fname = None
        try:
            con = db.getConnection()
            cur = con.cursor(cursor_factory=psycopg2.extras.DictCursor)
            f = None
            try:
                fname = tmpnam() # security risk: symlink attacks
                exportAsXLS(cur, fname, self.listDatasetIds(), case_parts, anonymize)
                f = open(fname, "rb")
                return f.read()
            finally:
                if f:
                    try: f.close()
                    except: pass
                if fname:
                    try: unlink(fname)
                    except: pass
        finally:
            db.recycleConnection(con, cur)


class CaseOverview:
    def __init__(self, search_str=None):
        self.ds_list = []
        self.sql = None

    def sortByLastname(self):
        def _cmp(a, b):
            if a is None and b is None: return 0
            if a is None: return -1
            if b is None: return +1
            return strcoll(a.last_name, b.last_name)
        self.ds_list.sort(_cmp)

    def search(self, search_str=None):
        self.ds_list = []
        search = CaseSearch()
        result = search.perform(search_str)
        self.sql = search.get_sql()
        factory = CaseFactory()
        for row in result:
            case = factory.loadByName( \
                row['id'], \
                row['phase'], \
                row['vorname_1'], \
                row['name_1'], \
                row['kundennummer'], \
                row['bearbeiter_id'], \
                row['einverstaendniserklaerung'], \
                row['s_id'], \
                row['status'], \
                row['zugriff'], \
                row['zeitraum_nachbetreuung'], \
                row['art_beendigung'])
            self.ds_list.append(case)
        return self.ds_list

    def numDatasets(self):
        return len(self.ds_list) 

    def getDatasets(self):
        return self.ds_list

    def get_sql(self):
        return self.sql

class CaseFactory:
    '''Factory for case object. This factory provides methods to either load
    existing cases from db or create new cases'''

    def loadById(self, id):
        '''Load a case with the given id. Return a case object'''
        case = Case()
        result = None
        conn, c = None, None
        try:
            conn = db.getConnection()
            c    = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
            fields = {'id': id}
            c.execute(LOAD_CASE_SQL, fields)
            result = c.fetchone()
            if not result: raise LoadCaseNotExistsError("Es konnte keine Fallakte geladen werden.") 
        finally:
            db.recycleConnection(conn, c)

        case.id = int(id)
        case.last_name     = unicode(str(result.get('name_1', '')), 'utf-8')
        case.first_name    = unicode(str(result.get('vorname_1', '')), 'utf-8')
        case.knr           = unicode(str(result.get('kundennummer', '')), 'utf-8')
        case.first_meeting = result.get('erstgespraech')
        case.cm_end        = result.get('beendigung_5')
        case.editor        = result.get('bearbeiter_id', None)
        case.state         = State(id)

        # Hack. These values are used to determine if the phase of CM can be startet
        case.zugang        = result.get('zugang_kompetenzagentur')
        case.letzteschul   = result.get('letzte_jahr_allg_schule')
        case.prognose      = result.get('prognose_liegt_vor')

        return case

    def loadByName(self, id, phase, first_name, last_name, knr, bearbeiter_id, einverst, s_id, s_state, s_access, s_cm_finished, s_cm_canceled):
        '''This method does not actually loads a case from the database. It
        just returns a case object initiated with the provided parameters'''
        case = Case()
        case.id = int(id)
        case.phase = int(phase)
        case.first_name = unicode(str(first_name), 'utf-8')
        case.last_name  = unicode(str(last_name),  'utf-8')
        case.knr        = unicode(str(knr),        'utf-8')
        case.editor     = bearbeiter_id
        case.state      = State()
        case.state.phase = phase
        case.state.setData(s_id, id, s_state, s_access, s_cm_finished, s_cm_canceled)
        case.privacy_statement_signed = einverst in (1, -1) and True or False
        return case

    def createNew(self, init=True, uuid=None):
        '''This method creates a new case in the database and takes care about
        creating the right dependencies for repeat groups. If 'init' is true
        will get initiated. This method returns the case object'''  
        conn, cur = None, None
        try:
            try:
                conn = db.getConnection()
                cur  = conn.cursor()
                cur.execute(CREATE_CASE_SQL, {'uuid': uuid})
                id = int(cur.fetchone()[0])
                conn.commit()
            finally:
                db.recycleConnection(conn, cur)
            # XXX: This a workaround the RGs
            self._createDependencies(id, uuid)
        except:
            traceback.print_exc(file=sys.stderr)
            return None
        if init == True:
            self._initNewCase(id)

        case = Case()
        case.id = id
        return case

    def _initNewCase(self, id):
        conn, cur = None, None
        try:
            user = session['USER_AUTHORIZED']
            agency = Agency()
            fields = {
                'ka_name':  agency.getName(), 
                'vorname':  user.first_name,
                'nachname': user.last_name,
                'fkz':      agency.getFKZ(), 
                'id':       int(id)
                }
            conn = db.getConnection()
            cur = conn.cursor()
            cur.execute(INIT_CASE_SQL, fields)
            conn.commit()
        finally:
            db.recycleConnection(conn, cur)

    def _createDependencies(self, id, uuid=None):
        con, cur = None, None
        try:
            con = db.getConnection()
            # create one Kompetenzfestellung
            cur = con.cursor()
            cur.execute(CREATE_RG_KOMPETENZ_DS, { 'id': id, 'uuid': uuid})
            komp_id = cur.fetchone()[0]
            cur.close(); cur = None

            # TODO: Uncomment the following as soon as the SP exist.
            # create three Maßnahmen per master data set
            #for i in range(3):
            #    cur = con.cursor()
            #    cur.execute(CREATE_RG_MASSNAHME_DS, { 'id': komp_id })
            #    cur.close(); cur = None

            # create three Beherrschungsprache per Kompetenzfestellung
            #for i in range(3):
            #    cur = con.cursor()
            #    cur.execute(CREATE_RG_SPRACHE_DS, { 'id': komp_id })
            #    cur.close(); cur = None

            # TODO: Uncomment the following as soon as the SP exist.
            # create three Maßnahmen per master data set
            #for i in range(3):
            #    cur = con.cursor()
            #    cur.execute(CREATE_RG_MASSNAHME_DS, { 'id': komp_id })
            #    cur.close(); cur = None

            con.commit()
        finally:
            db.recycleConnection(con, cur)

    def _createTablesForImport(self, relations, uuid=None):
        masterId, kompetenzId = None, None

        con, cur = None, None
        createdTables = {}
        try:
            con = db.getConnection()
            # a master dataset has to be created each time
            cur = con.cursor()

            # TODO: It seems that the following lines of code are doing already
            # implemented functionality. Think about cleaning up here but i do
            # not fully understand what is happening exctly here. Maybe the
            # next two outcommented lines could do the trick.   
            #case = self.creatNew()
            #masterId = case.id

            cur.execute(CREATE_CASE_SQL, {'uuid': uuid})
            masterId = cur.fetchone()[0]
            cur.close(); cur = None
            createdTables['master_tbl'] = masterId

            # kompetenz rg has to be created each time
            cur = con.cursor()
            cur.execute(CREATE_RG_KOMPETENZ_DS, { 'id': masterId, 'uuid': uuid })
            rid = cur.fetchone()[0]
            cur.close(); cur = None
            con.commit()

            createdTables['rg_kompetenzfestellung_tbl'] = rid

            for name, rels, separate in relations:
                id = None
                if not separate:
                    id = createdTables.get(name)

                if id is None:
                    try:
                        stmnt = CREATE_SQL[name]
                    except KeyError:
                        print >> sys.stderr, "Kann Statement zur Erzeugung von '%s' nicht finden." % name
                        continue
                    cur = con.cursor()
                    cur.execute(stmnt, { 'id': masterId, 'uuid': uuid })
                    id = cur.fetchone()[0]
                    cur.close(); cur = None

                # write the relations
                for rel in rels:
                    rel.sqlUpdate(con, id)

                if not separate:
                    createdTables[name] = id
        finally:
            db.recycleConnection(con, cur)

        return masterId

    def _fetchKundennummerFromPDF(self, tbls):
        for n, rels, sep in tbls:
            for rel in rels:
                try:
                    return rel.getColumn(u"kundennummer")
                except KeyError:
                    pass
        
        return None


    def _caseExists(self, knummer):
        con, cur = None, None
        try:
            con = db.getConnection()
            cur = con.cursor()
            cur.execute(CASE_EXISTS, { 'knummer': knummer })
            row = cur.fetchone()
            if row is None: return False
            return row[0]
        finally:
            db.recycleConnection(con, cur)


class CaseStandin:
    '''This class represents the standin for a case''' 
    def __init__(self, case_id=None):
        self.case_id = case_id
        self.group_ids = []
        self._loadFromDB()

    def _loadFromDB(self):
        fields = {'id': self.case_id}
        con, cur = None, None
        try:
            con = db.getConnection()
            cur = con.cursor(cursor_factory=psycopg2.extras.DictCursor)
            cur.execute(LOAD_STANDIN_SQL, fields)
            rows = cur.fetchall()
            for row in rows:
                self.group_ids.append(row['benutzer_id'])
        finally:
            db.recycleConnection(con, cur)

    def getGroups(self):
        '''Returns a list of group ids'''
        return self.group_ids

    def setGroups(self, groupid_list):
        '''Sets the Standin to the provided list of group ids'''
        old = Set(self.getGroups())
        new = Set(groupid_list)
        self._delete(old - new)
        self._add(new - old)

    def _delete(self, groupid_list):
        con, cur = None, None
        fields = {'caseid': self.case_id, 'groupid': None}
        try:
            con = db.getConnection()
            for uid in groupid_list:
                fields['groupid'] = uid
                cur = con.cursor()
                try:
                    cur.execute(DELETE_STANDIN_SQL, fields)
                except:
                    con.rollback()
                cur.close(); cur = None
            con.commit()
        finally:
            db.recycleConnection(con, cur)

    def _add(self, groupid_list):
        con, cur = None, None
        fields = {'caseid': self.case_id, 'groupid': None}
        try:
            con = db.getConnection()
            for uid in groupid_list:
                fields['groupid'] = uid
                cur = con.cursor()
                try:
                    cur.execute(ADD_STANDIN_SQL, fields)
                except:
                    con.rollback()
                cur.close(); cur = None
            con.commit()
        finally:
            db.recycleConnection(con, cur)

class CaseDigest:
    def __init__(self, id):
        self.resetValues(id)
        self._loadFromDB(id)

    def resetValues(self, id):
        self.id              = id
        self.first_name      = '' 
        self.last_name       = '' 
        self.knr             = '' 
        self.age             = '0'
        self.plz             = ''
        self.city            = ''
        self.street          = ''
        self.email           = ''
        self.telephone_1     = ''
        self.telephone_2     = ''
        self.telephone_more  = ''
        self.mobiletel       = ''
        self.statement       = False

        # Parents of the children
        self.parent_first_name  = '' 
        self.parent_last_name   = ''
        self.parent_telephone_1 = ''
        self.parent_telephone_2 = ''
        self.parent_mobiletel   = ''
        self.parent_email       = ''


    def hasSignedPrivacyStatement(self):
        return self.statement

    def _loadFromDB(self, id):
        try:
            result = None
            conn, c = None, None
            try:
                conn = db.getConnection()
                c    = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
                fields = {'id': id}
                c.execute(LOAD_CASE_DIGEST_SQL, fields)
                result = c.fetchone()
            finally:
                db.recycleConnection(conn, c)

            self.last_name         = unicode(str(result.get('name_1', '')), 'utf-8')
            self.first_name        = unicode(str(result.get('vorname_1', '')), 'utf-8')
            self.knr               = unicode(str(result.get('kundennummer')), 'utf-8')
            self.plz               = unicode(str(result.get('plz', '')), 'utf-8')
            self.street            = unicode(str(result.get('strasse', '')), 'utf-8')
            self.city              = unicode(str(result.get('wohnort', '')), 'utf-8')
            self.telephone_1       = unicode(str(result.get('vorwahl', '')), 'utf-8')
            self.telephone_2       = unicode(str(result.get('telefon_1', '')), 'utf-8')
            self.mobiletel         = unicode(str(result.get('mobiltelefon', '')), 'utf-8')
            self.telephone_more    = F.NA(unicode(str(result.get('telefon_sonstige', '')), 'utf-8')).replace('\n', '<br>')

            self.email             = unicode(str(result.get('email', '')), 'utf-8')
            self.age               = result.get('alter') and "%.0f" % result.get('alter') or "0"
            self.statement         = (result.get('einverstaendniserklaerung') >= 0)

            # Parents of the kids
            self.parent_first_name  = unicode(str(result.get('vorname_2', '')), 'utf-8')
            self.parent_last_name   = unicode(str(result.get('name_2', '')), 'utf-8')
            self.parent_telephone_1 = unicode(str(result.get('vorwahl_1', '')), 'utf-8')
            self.parent_telephone_2 = unicode(str(result.get('telefon_1_1', '')), 'utf-8')
            self.parent_mobiletel   = unicode(str(result.get('mobiltelefon_1', '')), 'utf-8')
            self.parent_email       = unicode(str(result.get('email_1', '')), 'utf-8')

            # Angaben zur Beendigung
            art_id         = result.get('art_beendigung', '')
            vermittlung_id = result.get('vermittlung', '')
            conn, c = None, None
            try:
                conn = db.getConnection()
                c    = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
                try:

                    fields = {'id': art_id}
                    c.execute(LOAD_ART_BEENDIGUNG, fields)
                    result = c.fetchone()
                    self.type_ending = unicode(str(result.get('value', '')), 'utf-8')

                    fields = {'id': vermittlung_id}
                    c.execute(LOAD_STATUS_BEENDIGUNG, fields)
                    result = c.fetchone()
                    self.state_ending = unicode(str(result.get('value', '')), 'utf-8')
                except:
                    self.type_ending = None
                    self.state_ending = None
            finally:
                db.recycleConnection(conn, c)

        except:
            traceback.print_exc(file=sys.stderr)

class SessionCase:
    '''This class holds all needed information for the currently selected case'''
    def __init__(self, id=None):
        if id is None: self.id = None
        else:          self.id = int(id)
        self.page_id           = None 
        self.mode              = "show" 
        self.form_errors       = {}
        self.last_name         = None
        self.first_name        = None
        self.knr               = None
        self.state             = None
        self.first_meeting     = None
        self.cm_end            = None
        self.privacy_statement = False

        self.street = None 
        self.plz = None 
        self.city = None

    def getState(self):
        return self.state

    def getEditor(self):
        return self.state

    def getMode(self):
        return self.mode

    def setMode(self, mode="show"):
        self.mode = mode

    def setPageId(self, id):
        self.page_id = id

    def getPageId(self):
        return self.page_id

    def getFormErrors(self):
        return self.form_errors

    def setFormErrors(self, errors):
        self.form_errors = errors

    def hasPrivacyStatement(self):
        return self.privacy_statement

    def setData(self, params):
        self.last_name  = save_getone(params, "name_1",    "")
        self.first_name = save_getone(params, "vorname_1", "")
        self.street     = save_getone(params, "strasse",   "")
        self.plz        = save_getone(params, "plz",       "")
        self.city       = save_getone(params, "wohnort",   "")

    def getFirstMeeting(self):
        return self.first_name

    def isYoungerThan(self, mydate):
        '''Return True if case was active relative to mydate.

        Case either started or finished after mdate, or no end date set.
        '''
        y,m,d = mydate.split('-')
        if self.first_meeting:
            if self.first_meeting > date(int(y), int(m), int(d)):
                return True
            elif self.cm_end and (self.cm_end <= date(int(y), int(m), int(d))
                                    and self.cm_end != UNKNOWN_DATE):
                return False
            else:
                return True
        return True

    def save():
        pass

class Case:
    def __init__(self):
        self.id            = None 
        self.first_name    = None 
        self.last_name     = None 
        self.knr           = None 
        self.editor        = None
        self.standin       = None 
        self.state         = None
        self.first_meeting = None
        self.cm_end        = None

        self.digest                   = None
        self.page                     = None
        self.documents                = []
        self.appointments             = []
        self.privacy_statement_signed = False
        self.privacy_statement        = None
        self.aidplan_statement        = None
        self.discretion_statement     = None
    
        # Hack. These values are used to check if the cm phase can be started
        self.letzteschul              = None
        self.prognose                 = None

    def delete(self):
        '''Deletes the case from data base. Returns True if deletion succseeds'''
        conn, cur = None, None
        try:
            try:
                conn = db.getConnection()
                cur = conn.cursor()
                cur.execute(DELETE_CASE_SQL, {'id': self.id})
                conn.commit()
                return True
            finally:
                db.recycleConnection(conn, cur)
        except:
            traceback.print_exc(file=sys.stderr)
        return False 

    def getMode(self):
        session = self.getSessionCase()
        return session.getMode()

    def setMode(self, mode="show"):
        s = self.getSessionCase(self.id)
        s.setMode(mode)
        s.save()

    def getState(self):
        return self.state

    def setState(self, value):
        self.state.setState(value)

    def getSessionCase(self):
        '''Returns a SessionCaseObject which is initiated with values from the current user'''
        session                   = SessionCase(self.id)
        session.last_name         = self.last_name
        session.first_name        = self.first_name 
        session.knr               = self.knr
        session.state             = self.state
        session.first_meeting     = self.first_meeting
        session.cm_end            = self.cm_end
        privacy_statement         = self.getPrivacyStatement()
        session.privacy_statement = privacy_statement.isSigned() 
        return session

    def getStandin(self):
        '''Returns the standin of the case'''
        if not self.standin is None: return self.standin
        try:
            self.standin = CaseStandin(self.id) 
            return self.standin
        except:
            self.standin = None
            raise
        return None

    def getEditor(self):
        '''Returns the editor of the case'''
        return UserObject(self.editor)

    def setEditor(self, userid):
        '''sets the editor of the case to the given userid'''
        con, curr = None, None
        user = UserObject(userid)
        fields1 = {'caseid': self.id, 'userid': userid}
        fields2 = {'caseid': self.id, 'firstname': user.first_name, 'lastname': user.last_name}
        try:
            con = db.getConnection()
            cur = con.cursor()
            try:
                cur.execute(SET_EDITOR_SQL, fields1)
                cur.execute(SET_EDITOR_NAME_SQL, fields2)
            except:
                con.rollback()
            con.commit()
        finally:
            db.recycleConnection(con, cur)
        return self.editor

    def getPage(self, datapage=None):
        '''Returns a datapage with the given name of the case'''
        if not self.page is None: 
            return self.page
        try:
            self.page = datapage
            self.page.setCaseId(self.id)
            return self.page
        except:
            self.page = None
            raise
        return None

    def getKompetenzId(self):
        '''Returns the id of the kompetenz repeat group'''
        con, cur = None, None
        id = None
        try:
            con = db.getConnection()
            # a master dataset has to be created each time
            cur = con.cursor(cursor_factory=psycopg2.extras.DictCursor)
            cur.execute(FETCH_KOMPETENZ_ID, { 'id': self.id })
            row = cur.fetchone()
            if row: 
                id = row['id']
            cur.close(); cur = None
        finally:
            db.recycleConnection(con, cur)
        return id 

    def getDigest(self, empty=False):
        '''Returns the digest of the case'''
        if not self.digest is None: return self.digest
        try:
            # Create digest
            self.digest = CaseDigest(self.id)
            # This is Ürkan-Mode
            if empty:
                self.digest.resetValues(self.id)
            return self.digest
        except:
            self.digest = None
            raise
        return None

    def getPrivacyStatement(self):
        '''Returns the privacy statement of the case'''
        if not self.privacy_statement is None: return self.privacy_statement
        try:
            # Create privacy_statement 
            self.privacy_statement = PrivacyStatement(self.id)
            self.privacy_statement.fillout(self.getDigest(), Agency()) 
            return self.privacy_statement
        except:
            self.privacy_statement = None
            raise
        return None

    def getDiscretionStatement(self):
        '''Returns the privacy statement of the case'''
        if not self.discretion_statement is None: return self.discretion_statement
        try:
            # Create discretion_statement 
            self.discretion_statement = DiscretionStatement(self.id)
            self.discretion_statement.fillout(self.getDigest(), Agency()) 
            return self.discretion_statement
        except:
            self.discretion_statement = None
            raise
        return None

    def getAidPlanStatement(self):
        '''Returns the aid plan statement of the case'''
        if not self.aidplan_statement is None: return self.aidplan_statement
        try:
            # Create aidplan statement 
            rg_list = AidList(self.id)
            self.aidplan_statement = AidPlanStatement(self.id) 
            self.aidplan_statement.fillout(client=self.getDigest(), agency=Agency(), rg_list=rg_list.getDatasets()) 
            return self.aidplan_statement
        except:
            self.aidplan_statement = None
            raise
        return None

    def getDocuments(self):
        '''Returns a list of documents which are attached to this case'''
        try:
            # Load list of documents 
            self.documents = listDocuments(self.id) 
            return self.documents
        except:
            self.documents = []
            raise
        return [] 

    def getAppointments(self):
        '''Returns a list of appointments which are attached to this case'''
        try:
            # Load list of appointments 
            pass
        except:
            self.documents = []
            raise
        return [] 

    def anonymize(self):
        '''Deleted personal data in the case'''
        conn, cur = None, None
        try:
            try:
                conn = db.getConnection()
                cur  = conn.cursor()
                d = { 'id': self.id }
                cur.execute(ANONYMIZE_CASE, d)
            finally:
                db.recycleConnection(conn, cur)
            state = self.getState()
            state.setState(5)
            return True
        except:
            traceback.print_exc(file=sys.stderr)
        return False

    def restore(self):
        #If case has no EE then set EE to undefined on restore
        ps = self.getPrivacyStatement()
        if not ps.isSigned():
            ps.keepUndefined()
        self.getState().setState(1)

    def getRequiredFields(self):
        con, cur = None, None
        try:
            con = db.getConnection()
            cur = con.cursor()
            rf = RequiredFields()
            rf.extractRequiredFields(g.formedTree)
            return rf.loadFromDatabase(self.id, cur) and rf or None
        finally:
            db.recycleConnection(con, cur)

    def getFirstMeeting(self):
        return self.first_name

    def isYoungerThan(self, mydate):
        '''Return True if case was active relative to mydate.

        Case either started or finished after mdate, or no end date set.
        '''
        y,m,d = mydate.split('-')
        if self.first_meeting:
            if self.first_meeting > date(int(y), int(m), int(d)):
                return True
            elif self.cm_end and (self.cm_end <= date(int(y), int(m), int(d))
                                    and self.cm_end != UNKNOWN_DATE):
                return False
            else:
                return True
        return True

class State(object):
    '''This class store the state and the last accesstime of the case.
    Possible states are:
        * Open (1)
        * Closes (2)
        * outstanding deletion (3)
        * outstanding anonymision (4)
        * anonymised (5)'''

    def getLastDateFromDB(self):
        if self._last_date:
            return self._last_date
        else:
            # Reload all the data. As fetching the last appointment
            # for the case is the most expensive part of the operation getting
            # some more values does not hurt.
            self._load()
            return self._last_date

    last_date = property(getLastDateFromDB)

    def __init__(self, master_id=None):
        self.master_id  = master_id
        self.id         = None
        self.state      = None
        self.access     = None
        self.phase      = None
        self._last_date = None

        if master_id: 
            self.master_id = master_id
            self._load()


    def setData(self, id, master_id, state, access, cm_finished, cm_canceled):
        self.master_id   = master_id
        self.id          = id
        self.state       = state
        self.access      = access
        self.cm_finished = cm_finished
        self.cm_canceled = cm_canceled

    def _load(self):
        conn, cur = None, None
        try:
            try:
                conn = db.getConnection()
                cur = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
                cur.execute(LOAD_CASESTATE_SQL, {'id': self.master_id})
                result = cur.fetchone()
                self.id          = result.get('id')
                self.state       = result.get('status')
                self.access      = result.get('zugriff')
                self._last_date  = result.get('letzer_termin')
                self.cm_finished = result.get('cm_beendet')
                self.cm_canceled = result.get('cm_abbruch')
                self.phase       = result.get('phase')
            finally:
                db.recycleConnection(conn, cur)
        except:
            traceback.print_exc(file=sys.stderr)
        return False 

    def getStateLabel(self):
        state_labels = {'1'   : _('case_state_label_open'),
                        '2'   : _('case_state_label_open'),
                        '3'   : _('case_state_label_outstanding_delete'),
                        '4'   : _('case_state_label_outstanding_anon'),
                        '5'   : _('case_state_label_anon')
                      }
        return state_labels.get(str(self.state))

    def getState(self):
        return self.state

    def isCmFinished(self):
        return self.cm_finished != -1

    def isCmCanceled(self):
        return self.cm_canceled == 1

    def setState(self, value):
        conn, cur = None, None
        try:
            try:
                conn = db.getConnection()
                cur = conn.cursor()
                fields =  {'status': value, 'id': self.id}
                cur.execute(SET_CASESTATE_SQL, fields)
                conn.commit()
                self.state = value
            finally:
                db.recycleConnection(conn, cur)
            return True
        except:
            traceback.print_exc(file=sys.stderr)
        return False 

    def setPhase(self, phase):
        conn, cur = None, None
        try:
            try:
                conn = db.getConnection()
                cur = conn.cursor()
                fields =  {'phase': phase, 'id': self.id}
                cur.execute(SET_CASEPHASE_SQL, fields)
                conn.commit()
                self.phase = phase
            finally:
                db.recycleConnection(conn, cur)
            return True
        except:
            traceback.print_exc(file=sys.stderr)
        return False 

    def getPhase(self):
        return self.phase

    def getPhaseDescription(self):
        return phase_description(self.phase)

    def phasesAreConsistent(self):
        '''Returns True when the required fields for the current phase are
        complete and there are no other inconsistencys. Else False'''
        case = CaseFactory().loadById(self.master_id) # Ugly we need to load the case here :(
        rf = case.getRequiredFields()
        cf = self.getPhase()
        p_pairs = phases_pairs(rf, cf)
        for pair in p_pairs:
            p1, p2 = pair.getStart(), pair.getEnd()
            # Find current phase
            if p1.isRunning() and not p1.isRecursiveComplete():
                return False
            elif p2.isRunning() and not p2.isRecursiveComplete():
                return False 
        # Check other inconsistency
        # Search all cases with are inconsistent and then check if the current case is among them
        user = session['USER_AUTHORIZED']
        search_str = "state:1;state:2;state:4;state:5;bad:1;bad:2;bad:3;bad:4;bad:5;bad:6;bad:7;own:%s;standin:%s" % (user.id, user.id)
        error_cases = Set(c.id for c in CaseOverview().search(search_str))
        if case.id in error_cases: return False
        return True

    def phasesAreCompleted(self):
        '''Returns True if the required fields for the ending of the 
        current phase pair are completed and the phase ist finished. 
        Else False.'''
        case = CaseFactory().loadById(self.master_id) # Ugly we need to load the case here :(
        rf = case.getRequiredFields()
        cf = self.getPhase()
        current_phase = currentPhase(rf, cf)

        # Check other inconsistency
        # Search all cases with are inconsistent and then check if the current case is among them
        user = session['USER_AUTHORIZED']
        search_str = "state:1;state:2;state:4;state:5;bad:1;bad:2;bad:3;bad:4;bad:5;bad:6;bad:7;own:%s;standin:%s" % (user.id, user.id)
        error_cases = Set(c.id for c in CaseOverview().search(search_str))
        if case.id in error_cases: return False

        # No Anonymization for Clearing Phase
        if cf == CLEAR_START or cf == CLEAR_ENDE:
            return False
        elif phase_end(cf) and current_phase.isRecursiveComplete():
            return True
        else:
            return False 


    def getAccessTime(self, format=None):
        if format:
            try:
                return self.access.strftime(format)
            except:
                return dd_mm_YYYY(self.access)
        return self.access

    def getLastDate(self, format=None):
        '''Returns the last occouring date from appointments, aids, or the last accesstime'''
        if format:
            try:
                return self.last_date.strftime(format)
            except:
                return dd_mm_YYYY(self.last_date)
        return self.last_date

    def setAccessTime(self):
        pass

    def getMaxSavetime(self):
        agency  = Agency()
        return agency.getMaxSavetime()

    def daysToMaxSavetime(self):
        maxage  = self.getMaxSavetime()
        if self.isCmFinished() or self.isCmCanceled():
            age     = datetime.now() - self.getAccessTime()
        else:
            age     = datetime.now() - self.getLastDate()
        if int(maxage) != 0:
            return int(maxage) - age.days
        return None

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
